/*
 * Copyright (c) 2012 TopCoder, Inc. All rights reserved.
 * 
 */

/**
 * This script file encapsulates the JS to present Bing Map and search FARS
 * accidents data. It is included in farsBingMap.jap.
 * <p>
 * It provides two public functions:<br>
 * 1. To search accidents near address with a mileage radius<br>
 * 2. To search accidents near route by given start-end addresses
 * </p>
 * Note the JS only sends requests to the Bing Map/Bing Spatial Data Services.
 * It does not request back to this application again.
 * 
 * @author liuliquan, hanno
 * @version 1.0
 */

/**
 * The maximum number of bing responses cached locally.
 */
var MAX_BING_CACHE_SIZE = 25;

/**
 * Initialize the Bing map.
 * 
 * @constructor
 * @param {Object}
 *            dataSource - the data source
 * @param {String}
 *            mapDivId - the HTML DIV id of the map
 */
function FARSBingMap (dataSource, mapDivId) {
    /**
     * The key used to query Bing Map and Bing Spatial Data Services.
     * 
     * Initialized in constructor and will never change.
     */
    var queryKey;

    /**
     * The data source url used to access Bing Spatial Data Services to search
     * FARS accident data.
     * 
     * Initialized in constructor and will never change.
     */
    var dataSourceUrl;

    /**
     * The entity type name used to access Bing Spatial Data Services to search
     * FARS accident data.
     * 
     * Initialized in constructor and will never change.
     */
    var entityTypeName;

    /**
     * The Bing map.
     * 
     * Initialized in constructor and will never change.
     */
    var map;

    /**
     * The manager used to find route from user input start address to end
     * address.
     * 
     * Initialized in constructor and will never change.
     */
    var directionsManager;

    /**
     * The manager used to find geo location for user input address.
     * 
     * Initialized in constructor and will never change.
     */
    var searchManager;

    /**
     * This variable records the current address shown on map.
     * 
     * Set in onAddressFound function. Cleared in clearCurrentAddress function.
     */
    var currentAddress;

    /**
     * This variable records the collection of FARS accident pinpoints on map.
     * The items are Microsoft.Maps.Pushpin objects.
     * 
     * Items are added in showAccidentsNearAddress and showAccidentsNearRoute
     * function. Cleared in clearAccidentInfo function.
     */
    var accidentPinPoints;

    /**
     * This variable caches responses from the spatial service.
     */
    var bingCache;

    /**
     * The current size of the bing cache.
     */
    var bingCacheSize;

    /**
     * Returns the cached value for the given key or null if not found. The
     * cache timestamp of the found value will be refreshed.
     * 
     * @param {Object}
     *            key - the key to lookup
     */
    var getCache = function (key) {
        var entry = bingCache[key];
        if (entry) {
            entry.timestamp = new Date();
            return entry.value;
        }
        return null;
    }

    /**
     * Puts the value into the cache.
     * 
     * @param {Object}
     *            key - the key. May be null or undefined in which case nothing
     *            is put to cache
     * @param {Object}
     *            value - the value
     */
    var putCache = function (key, value) {
        if (!key) {
            return;
        }

        if (bingCacheSize < MAX_BING_CACHE_SIZE) {
            bingCacheSize++;
        } else {
            var oldest;
            for ( var e in bingCache) {
                // find the oldest entry in the cache
                if (bingCache.hasOwnProperty(e) && (!oldest || e.timestamp < oldest.timestamp)) {
                    oldest = e;
                }
            }

            // remove the oldest entry
            delete bingCache[oldest];
        }

        bingCache[key] = {
            key : key,
            value : value,
            timestamp : new Date()
        };
    };

    /**
     * Clear current address, remove it from map.
     * 
     */
    var clearCurrentAddress = function () {
        if (currentAddress) {
            map.entities.remove(currentAddress);
            currentAddress = null;
        }
    };

    /**
     * Clear the current route on map.
     * 
     */
    var clearCurrentRoute = function () {
        if (directionsManager) {
            directionsManager.resetDirections();
        }
    };

    /**
     * Clear accident info on map.
     * 
     */
    var clearAccidentInfo = function () {
        if (accidentPinPoints) {
            map.entities.remove(accidentPinPoints);
            accidentPinPoints.clear();
        }
    };

    /**
     * Create direction manager.
     * 
     */
    var createDirectionsManager = function () {
        if (!directionsManager) {
            directionsManager = new Microsoft.Maps.Directions.DirectionsManager(map);
        }

        Microsoft.Maps.Events.addHandler(directionsManager, 'directionsError', function (arg) {
            alert('Failed to find route: ' + arg.message);
        });

        Microsoft.Maps.Events.addHandler(directionsManager, 'directionsUpdated', function () {
            clearAccidentInfo();
            onRouteFound();
        });
    };

    /**
     * Create search manager.
     */
    var createSearchManager = function () {
        if (!searchManager) {
            searchManager = new Microsoft.Maps.Search.SearchManager(map);
        }
    };

    /**
     * The call back method which is invoked when address is found.
     * 
     * It pinpoints the address onto map and searches the nearby accidents.
     * 
     * 
     * @param {Object}
     *            result - The geo location result of address
     * @param {Integer}
     *            miles - The radius distance from the address which defines the
     *            area to search accidents
     */
    var onAddressFound = function (result, miles) {
        // Get address location:
        var location = result && result.results && result.results[0] && result.results[0].location;

        if (!location) {
            alert('No address found');
            return;
        }

        // Create a draggable pushpin for the address
        clearCurrentAddress();
        currentAddress = new Microsoft.Maps.Pushpin(location, {
            draggable : true,
            icon : 'i/addressPoint.png'
        });

        // Add drag end event handler for the pushpin to search accidents
        Microsoft.Maps.Events.addHandler(currentAddress, 'dragend', function (e) {
            clearAccidentInfo();
            searchAccidentsNearAddress(e.entity.getLocation(), miles);
        });

        // Show the currentAddress on map
        map.entities.push(currentAddress);
        map.setView({
            center : location
        });

        // Search nearby accidents
        searchAccidentsNearAddress(location, miles);
    };

    /**
     * Use Bing Spatial Data Services to search the nearby accidents of given
     * location within given miles distance.
     * 
     * @param {Object}
     *            location - The geo location of address
     * @param {Integer}
     *            miles - The radius distance from the address which defines the
     *            area to search accidents
     */
    var searchAccidentsNearAddress = function (location, miles) {
        var cacheKey = 'sna(' + location.latitude + "," + location.longitude + "," + miles + ')';
        var cachedValue = getCache(cacheKey);

        if (cachedValue) {
            showAccidentsNearAddress(cachedValue);
        } else {
            var requestStr = dataSourceUrl + "/" + entityTypeName + "?spatialFilter=nearby(" + location.latitude + ","
                    + location.longitude + "," + (miles * 1.609344)
                    + ")&$select=Latitude,Longitude,AccidentType&$format=json&key=" + queryKey
                    + "&Jsonp=showAccidentsNearAddress&jsonso=" + cacheKey;

            callSpatialService(requestStr);
        }
    };

    /**
     * This method is invoked when direction calculation is done, and the route
     * has been shown on map.
     * 
     * It search the accidents near each segment of route splitted by way
     * points.
     * 
     */
    var onRouteFound = function () {
        var wayPoints = directionsManager.getAllWaypoints();
        var i;

        // For each segment between way points, search accidents near route
        for (i = 0; i < wayPoints.length - 1; i++) {
            searchAccidentsNearRoute(wayPoints[i].getLocation(), wayPoints[i + 1].getLocation());
        }
    };

    /**
     * Use Bing Spatial Data Services to search the accidents near the route
     * which is defined by given start and end address.
     * 
     * @param {Object}
     *            start - The geo location of start address
     * @param {Object}
     *            end - The geo location of end address
     */
    var searchAccidentsNearRoute = function (start, end) {
        var cacheKey = 'snr(' + start.latitude + "," + start.longitude + "," + end.latitude + "," + end.longitude + ')';
        var cachedValue = getCache(cacheKey);

        if (cachedValue) {
            showAccidentsNearRoute(cachedValue);
        } else {
            var requestStr = dataSourceUrl + "/" + entityTypeName + "?spatialFilter=nearroute('" + start.latitude + ","
                    + start.longitude + "','" + end.latitude + "," + end.longitude
                    + "')&$select=Latitude,Longitude,AccidentType&$format=json&key=" + queryKey
                    + "&Jsonp=showAccidentsNearRoute&jsonso=" + cacheKey;

            callSpatialService(requestStr);
        }
    };

    /**
     * Build HTML content for accident type.
     * 
     * @param {String}
     *            accidentType - The accident type. It is a 5-length string with
     *            each char either 1 or 0. For example, the value 11000 means
     *            the accident involves Pedestrian and Cyclist.
     * 
     */
    var buildHtmlContentForAccidentType = function (accidentType) {
        var htmlContent = '<table>';
        var i;

        for (i = 0; i < accidentType.length; i++) {
            if (accidentType.charAt(i) === '1') {
                htmlContent += '<td><img src="i/icon' + (i + 1) + '.png"/></td>';
            }
        }

        return htmlContent + '</table>';
    };

    /**
     * Create a script to be executed with src as given request string.
     * 
     * 
     * @param {String}
     *            request - The request string to Bing Spatial Data Services.
     */
    var callSpatialService = function (request) {
        var script = document.createElement('script');
        script.setAttribute('type', 'text/javascript');
        script.setAttribute('src', request);
        document.body.appendChild(script);
    };

    /**
     * Find address geo location, and search the accidents near the address.
     * 
     * 
     * @param {String}
     *            address - The address to find. Can be zip code, address
     *            name...
     * @param {Integer}
     *            miles - The radius distance from the address which defines the
     *            area to search accidents
     */
    this.findAddress = function (address, miles) {
        if (!address || address == 'Search Term') {
            alert('Please enter a search term');
            return;
        }

        // 1. Clear the map:
        clearAccidentInfo();
        clearCurrentAddress();
        clearCurrentRoute();

        // 2. Build the request to find address:
        var request = {
            where : address,
            callback : onAddressFound,
            errorCallback : function (arg) {
                alert('Failed to find address: ' + arg.message);
            },
            count : 1, // we process one result only anyway
            userData : miles
        };

        // 3. Do search:
        searchManager.geocode(request);
    };

    /**
     * Show the accidents near route.
     * 
     * @param {Object}
     *            response - The Bing Spatial Data Services response object
     *            which contains the accidents data found.
     * @param {Object}
     *            so - state object
     */
    window.showAccidentsNearRoute = function (response, so) {
        // cache the response locally
        putCache(so, response);

        // 1. Get the nearby search results of accidents:
        var results = response.d.results;
        var i;

        // 2. Create a pushpin for each accident:
        for (i = 0; i < results.length; i++) {
            // Build html context for accident type
            var htmlContent = buildHtmlContentForAccidentType(results[i].AccidentType);
            // Create a pushpin for accident
            var location = new Microsoft.Maps.Location(results[i].Latitude, results[i].Longitude);
            accidentPinPoints.push(new Microsoft.Maps.Pushpin(location, {
                htmlContent : htmlContent,
                icon : "dummy"
            }));
        }
        map.entities.push(accidentPinPoints);
    };

    /**
     * Show the accidents near address.
     * 
     * @param {Object}
     *            response - The Bing Spatial Data Services response object
     *            which contains the accidents data found.
     * @param {Object}
     *            so - state object
     */
    window.showAccidentsNearAddress = function (response, so) {
        // cache the response locally
        putCache(so, response);

        // 1. Get the nearby search results of traffic accidents:
        var results = response.d.results;

        // 2. If current address is null, or if the results do not contain data,
        // return:
        if (!currentAddress || !results || !results[0]) {
            return;
        }

        // 3. Initialize bounds for map area:
        var minLatitude = results[0].Latitude;
        var maxLatitude = results[0].Latitude;
        var minLongitude = results[0].Longitude;
        var maxLongitude = results[0].Longitude;
        var i;

        // 4. Create a pushpin for each accident:
        for (i = 0; i < results.length; i++) {
            // Build html context for accident type
            var htmlContent = buildHtmlContentForAccidentType(results[i].AccidentType);
            // Create a pushpin for accident
            var loc = new Microsoft.Maps.Location(results[i].Latitude, results[i].Longitude);
            var pushpin = new Microsoft.Maps.Pushpin(loc, {
                htmlContent : htmlContent,
                icon : "dummy"
            });
            // Pinpoint to map
            accidentPinPoints.push(pushpin);

            // Determine bounds that display all pushpins
            if (results[i].Latitude > maxLatitude) {
                maxLatitude = results[i].Latitude;
            } else if (results[i].Latitude < minLatitude) {
                minLatitude = results[i].Latitude;
            }
            if (results[i].Longitude > maxLongitude) {
                maxLongitude = results[i].Longitude;
            } else if (results[i].Longitude < minLongitude) {
                minLongitude = results[i].Longitude;
            }
        }

        map.entities.push(accidentPinPoints);

        // 5. Set map view:
        // Add padding to map area so that pushpins are not on the edge
        var mapArea = Microsoft.Maps.LocationRect.fromLocations(new Microsoft.Maps.Location(minLatitude - 0.0001,
                minLongitude - 0.0001), new Microsoft.Maps.Location(maxLatitude + 0.0001, maxLongitude + 0.0001));

        map.setView({
            center : currentAddress.getLocation(),
            bounds : mapArea
        });
    };

    /**
     * Find route from given start address to end address.
     * 
     * @param {String}
     *            start - The start address
     * @param {String}
     *            end - The end address
     */
    this.findRoute = function (start, end) {
        if (!start || start == 'Start' || !end || end == 'End') {
            alert('Please enter a Start and End for the route');
            return;
        }

        // 1. Clear the map:
        clearAccidentInfo();
        clearCurrentAddress();
        clearCurrentRoute();

        // 2. Set driving mode:
        directionsManager.setRequestOptions({
            routeMode : Microsoft.Maps.Directions.RouteMode.driving
        });

        // 3. Set start point and end point:
        directionsManager.addWaypoint(new Microsoft.Maps.Directions.Waypoint({
            address : start
        }));
        directionsManager.addWaypoint(new Microsoft.Maps.Directions.Waypoint({
            address : end
        }));

        Microsoft.Maps.Events.addHandler(directionsManager.getAllWaypoints()[0], 'geocoded', function (arg) {
            if (!arg.waypoint.getLocation()) {
                alert('Start of route not found');
            }
        });

        Microsoft.Maps.Events.addHandler(directionsManager.getAllWaypoints()[1], 'geocoded', function (arg) {
            if (!arg.waypoint.getLocation()) {
                alert('End of route not found');
            }
        });

        // 4. Calculate directions:
        directionsManager.calculateDirections();

    };

    // 1. Set queryKey, dataSourceUrl, entityTypeName properties.
    queryKey = dataSource.queryKey;
    dataSourceUrl = dataSource.dataSourceUrl;
    entityTypeName = dataSource.entityTypeName;

    // 2. Create Map:
    Microsoft.Maps.loadModule('Microsoft.Maps.Themes.BingTheme', {
        callback : function () {
            var theme = new Microsoft.Maps.Themes.BingTheme();
            map = new Microsoft.Maps.Map(document.getElementById(mapDivId), {
                credentials : queryKey,
                showBreadcrumb : true,
                theme : theme,
                showMapTypeSelector : false,
                enableSearchLogo : false
            });
        }
    });

    // 3. Load Directions module, and create DirectionsManager after module
    // loaded:
    Microsoft.Maps.loadModule('Microsoft.Maps.Directions', {
        callback : createDirectionsManager
    });

    // 4. Load Search module, and create SearchManager after module loaded:
    Microsoft.Maps.loadModule('Microsoft.Maps.Search', {
        callback : createSearchManager
    });

    // 5. Initialize accidentPinPoints:
    accidentPinPoints = new Microsoft.Maps.EntityCollection();

    // 6. Initialize cache
    bingCache = {};
    bingCacheSize = 0;
}