/*
 * Copyright (C) 2012 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.bingmap.mashup.controller.fars;

import java.io.IOException;
import java.io.InputStream;
import java.text.MessageFormat;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Locale;

import javax.annotation.PostConstruct;

import org.apache.commons.io.IOUtils;
import org.apache.log4j.Logger;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.multipart.MultipartFile;

import com.topcoder.bingmap.spatial.datasource.BingSpatialConfigurationException;
import com.topcoder.bingmap.spatial.datasource.BingSpatialData;
import com.topcoder.bingmap.spatial.datasource.BingSpatialDataLoadType;
import com.topcoder.bingmap.spatial.datasource.BingSpatialDataParser;
import com.topcoder.bingmap.spatial.datasource.BingSpatialDataParsingException;
import com.topcoder.bingmap.spatial.datasource.BingSpatialDataSourceManagementException;
import com.topcoder.bingmap.spatial.datasource.BingSpatialDataSourceManager;
import com.topcoder.bingmap.spatial.datasource.BingSpatialDataSourceStatus;

/**
 * This controller is used by application administrator to upload FARS data to
 * Bing Spatial DataSource.
 * <p>
 * All the urls in this controller are protected by leveraging Spring Security,
 * including /upload.do and /uploadResult.do. They can only be visited by
 * authorized administrator.
 * </p>
 * <p>
 * Thread Safety: The ultimate thread safety depends on the thread safety of
 * Bing Spatial DataSource Management API. Without the insurance of thread
 * safety confirmed by Bing, the administrator should upload the data carefully.
 * However since the FARS data is uploaded yearly to spatial data source in
 * practice, so this should not be a problem.
 * </p>
 *
 * @author liuliquan, hanno
 * @version 1.0
 */
@Controller
public class FARSBingSpatialDataUploadController {
    /**
     * Class logger.
     */
    private static final Logger LOGGER = Logger.getLogger(FARSBingSpatialDataUploadController.class);

    /**
     * CSV suffix.
     */
    private static final String CSV = "csv";

    /**
     * Text plain MIME type.
     */
    private static final String TEXT_PLAIN = "text/plain";

    /**
     * XML suffix.
     */
    private static final String XML = "xml";

    /**
     * Text XML MIME type.
     */
    private static final String TEXT_XML = "text/xml";

    /**
     * The configured data parser.
     *
     * This field will be injected by Spring automatically.
     */
    private BingSpatialDataParser dataParser;

    /**
     * The configured data source manager.
     *
     * This field will be injected by Spring automatically.
     */
    private BingSpatialDataSourceManager dataSourceManager;

    /**
     * Empty constructor for controller class.
     */
    public FARSBingSpatialDataUploadController() {
        // empty
    }

    /**
     * Check bean is configured correctly.
     *
     * @throws BingSpatialConfigurationException
     *             if the bean is not configured correctly
     */
    @PostConstruct
    protected void checkInit() {
        if (this.dataParser == null) {
            throw new BingSpatialConfigurationException("dataParser is not set.");
        }
        if (this.dataSourceManager == null) {
            throw new BingSpatialConfigurationException("dataSourceManager is not set.");
        }
    }

    /**
     * Show the initial page to upload FARS data.
     *
     * @return Return the initial upload view
     */
    @RequestMapping(method = RequestMethod.GET, value = "/upload")
    public String showUpload() {
        return "upload";
    }

    /**
     * Upload FARS data to Bing Spatial DataSource.
     * <p>
     * Currently we support excel, csv and xml formats. For excel file the
     * application will parse it at first before uploading to Bing Spatial
     * DataSource. For csv and xml formats, the application assumes they are
     * already properly formatted and will upload them directly without parsing.
     * </p>
     *
     * @param uploadFile
     *            The file uploaded by user
     * @param uploadType
     *            The user specified upload type
     * @return the redirection to /uploadResult.do for successful upload. Note
     *         the upload job is just successfully created and may not be
     *         completed though.
     * @throws BingSpatialDataSourceManagementException
     *             if any data source error occurs
     * @throws BingSpatialDataParsingException
     *             if any parsing error occurs
     */
    @RequestMapping(method = RequestMethod.POST, value = "/upload")
    public String uploadSpatialData(MultipartFile uploadFile, BingSpatialDataLoadType uploadType)
        throws BingSpatialDataSourceManagementException, BingSpatialDataParsingException {
        BingSpatialData data = null;
        try {
            LOGGER.info(MessageFormat.format("{0} upload of {1}", uploadType, uploadFile.getOriginalFilename()));

            data = getData(uploadFile);
            this.dataSourceManager.loadData(data, uploadType);
            return "redirect:/uploadResult";
        } catch (BingSpatialDataSourceManagementException e) {
            LOGGER.error("Could not upload data: " + e.getMessage(), e);
            throw e;
        } catch (BingSpatialDataParsingException e) {
            LOGGER.error("Could not parse data: " + e.getMessage(), e);
            throw e;
        } finally {
            if (data != null) {
                IOUtils.closeQuietly(data.getData());
            }
        }
    }

    /**
     * Gets the BingSpatialData from the uploaded file.
     *
     * @param uploadFile
     *            the file
     * @return the BingSpatialData from the uploaded file.
     * @throws BingSpatialDataParsingException
     *             if any error occurs
     */
    private BingSpatialData getData(MultipartFile uploadFile) throws BingSpatialDataParsingException {
        try {
            InputStream fin = uploadFile.getInputStream();

            String name = uploadFile.getOriginalFilename().toLowerCase();
            if (name.endsWith(CSV)) {
                return new BingSpatialData(fin, CSV, TEXT_PLAIN);
            } else if (name.endsWith(XML)) {
                return new BingSpatialData(fin, XML, TEXT_XML);
            }

            // Otherwise the FARS data file must be an excel file, try to parse
            return this.dataParser.parseData(fin);
        } catch (IOException e) {
            String msg = "Could not get input: " + e.getMessage();
            LOGGER.error(msg, e);
            throw new BingSpatialDataParsingException(msg, e);
        }
    }

    /**
     * Show the FARS data upload result page.
     *
     * @return the upload result view
     */
    @RequestMapping(method = RequestMethod.GET, value = "/uploadResult")
    public String showUploadResult() {
        return "uploadResult";
    }

    /**
     * Invoked through AJAX by upload result page to poll the load status.
     *
     * @return the BingSpatialDataSourceStatus, it will be automatically
     *         converted to JSON by Spring
     */
    @RequestMapping(method = RequestMethod.GET, value = "/uploadStatus")
    @ResponseBody
    public BingSpatialDataSourceStatus getUploadStatus() {
        BingSpatialDataSourceStatus status;

        try {
            status = this.dataSourceManager.checkDataSourceStatus();

            status.setCompletedDateFormatted(formatDate(status.getCompletedDate()));
            status.setCreatedDateFormatted(formatDate(status.getCreatedDate()));
            LOGGER.info("Current status of upload: " + status.getStatus());
        } catch (BingSpatialDataSourceManagementException e) {
            String msg = "Could not get upload status: " + e.getMessage();
            LOGGER.error(msg, e);
            status = new BingSpatialDataSourceStatus();
            status.setStatus("ServerError");
            status.setErrorMessage(msg);
        }

        return status;
    }

    /**
     * Format the date in US locale.
     *
     * @param d
     *            date
     * @return the date in US locale.
     */
    private String formatDate(Date d) {
        if (d == null) {
            return "";
        }

        return new SimpleDateFormat("d MMM yyyy HH:mm:ss", Locale.US).format(d);
    }

    /**
     * Setter of dataParser.
     *
     * <p>
     * Called by Spring for injection.
     * </p>
     *
     * @param dataParser
     *            the data parser
     */
    public void setDataParser(BingSpatialDataParser dataParser) {
        this.dataParser = dataParser;
    }

    /**
     * Setter of dataSourceManager.
     *
     * <p>
     * Called by Spring for injection.
     * </p>
     *
     * @param dataSourceManager
     *            the data source manager
     */
    public void setDataSourceManager(BingSpatialDataSourceManager dataSourceManager) {
        this.dataSourceManager = dataSourceManager;
    }
}
