/*
 * Copyright (C) 2012 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.bingmap.spatial.datasource;

import java.io.InputStream;

import org.apache.log4j.Logger;

/**
 * This class represents the Bing Spatial data which can be uploaded to the
 * spatial data source.
 *
 * <p>
 * <b>Thread Safety:</b> This class is thread safe since it is immutable but the
 * contained input stream is not.
 * </p>
 *
 * @author liuliquan, hanno
 * @version 1.0
 */
public class BingSpatialData {
    /**
     * Logger.
     */
    private static final Logger LOGGER = Logger.getLogger(BingSpatialData.class);

    /**
     * The input stream of data. Set in constructor.
     */
    private final InputStream data;

    /**
     * The format of data, e.g. csv. Set in constructor.
     */
    private final String format;

    /**
     * The mimeType of data, e.g. text/plain. Set in constructor.
     */
    private final String mimeType;

    /**
     * Creates a new instance with given data, format and mimeType.
     *
     * @param data
     *            the spatial data
     * @param format
     *            the data format, e.g. csv
     * @param mimeType
     *            The data's mimeType, e.g. text/plain
     * @throws IllegalArgumentException
     *             if data is null or any of the string parameters is null or
     *             empty
     */
    public BingSpatialData(InputStream data, String format, String mimeType) {
        try {
            this.data = BingSpatialHelper.checkValidity("Data", data);
            this.format = BingSpatialHelper.checkValidity("Format", format);
            this.mimeType = BingSpatialHelper.checkValidity("MIME type", mimeType);
        } catch (IllegalArgumentException e) {
            LOGGER.error(e.getMessage(), e);
            throw e;
        }
    }

    /**
     * Getter of data.
     *
     * @return the data
     */
    public InputStream getData() {
        return this.data;
    }

    /**
     * Getter of format.
     *
     * @return the format
     */
    public String getFormat() {
        return this.format;
    }

    /**
     * Getter of mimeType.
     *
     * @return the mimeType
     */
    public String getMimeType() {
        return this.mimeType;
    }
}
