/*
 * Copyright (C) 2012 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.bingmap.spatial.datasource;

import javax.annotation.PostConstruct;

/**
 * This class represents the Bing Spatial Data Source with the necessary
 * information in order to access it.
 * <p>
 * The dataSourceName, entityTypeName, masterKey, queryKey, dataSourceUrlFile
 * are configuration entries and injected by Spring.
 * </p>
 * <p>
 * Please note that there are some restrictions on the dataSourceName:
 * <ul>
 * <li>can have up to 50 characters</li>
 * <li>can contain alphanumeric characters</li>
 * <li>can contain any of the following: ~ ` ! $ ^ _ = { }</li>
 * </ul>
 * </p>
 * <p>
 * The dataSourceUrl is set when upload to data source completed.
 * </p>
 * <p>
 * <b>Thread Safety:</b> This class is not thread safe since dataSourceUrl field
 * can be modified.
 * </p>
 *
 * @author liuliquan, hanno
 * @version 1.0
 */
public class BingSpatialDataSource {
    /**
     * Regular expression for allowed data source names.
     */
    private static final String DS_NAME_REGEX = "[a-zA-Z0-9~`!$^_={}]{1,50}";

    /**
     * The configured data source name.
     *
     * This field will be injected by Spring automatically.
     */
    private String dataSourceName;

    /**
     * The configured entity type name.
     *
     * This field will be injected by Spring automatically.
     */
    private String entityTypeName;

    /**
     * The configured master key of data source.
     *
     * This field will be injected by Spring automatically.
     */
    private String masterKey;

    /**
     * The configured query key of data source.
     *
     * This field will be injected by Spring automatically.
     */
    private String queryKey;

    /**
     * The configured path of file to store the URL to query data source.
     *
     * This field will be injected by Spring automatically.
     */
    private String dataSourceUrlFile;

    /**
     * The URL used to query Bing Spatial Data Source.
     */
    private String dataSourceUrl;

    /**
     * Empty constructor.
     */
    public BingSpatialDataSource() {
        // empty
    }

    /**
     * Check bean is configured correctly.
     *
     * @throws BingSpatialConfigurationException
     *             if the bean is not configured correctly
     */
    @PostConstruct
    protected void checkInit() {
        BingSpatialHelper.checkBeanConfig("data source name", this.dataSourceName);
        BingSpatialHelper.checkBeanConfig("data source URL file", this.dataSourceUrlFile);
        BingSpatialHelper.checkBeanConfig("Entity type name", this.entityTypeName);
        BingSpatialHelper.checkBeanConfig("Master key", this.masterKey);
        BingSpatialHelper.checkBeanConfig("Query key", this.queryKey);

        if (!this.dataSourceName.matches(DS_NAME_REGEX)) {
            throw new BingSpatialConfigurationException("The data source name does not match the regular expression "
                + DS_NAME_REGEX);
        }
    }

    /**
     * Returns the dataSourceName.
     *
     * @return the dataSourceName
     */
    public String getDataSourceName() {
        return dataSourceName;
    }

    /**
     * Sets the dataSourceName.
     *
     * @param dataSourceName
     *            the new dataSourceName to set
     */
    public void setDataSourceName(String dataSourceName) {
        this.dataSourceName = dataSourceName;
    }

    /**
     * Returns the entityTypeName.
     *
     * @return the entityTypeName
     */
    public String getEntityTypeName() {
        return entityTypeName;
    }

    /**
     * Sets the entityTypeName.
     *
     * @param entityTypeName
     *            the new entityTypeName to set
     */
    public void setEntityTypeName(String entityTypeName) {
        this.entityTypeName = entityTypeName;
    }

    /**
     * Returns the masterKey.
     *
     * @return the masterKey
     */
    public String getMasterKey() {
        return masterKey;
    }

    /**
     * Sets the masterKey.
     *
     * @param masterKey
     *            the new masterKey to set
     */
    public void setMasterKey(String masterKey) {
        this.masterKey = masterKey;
    }

    /**
     * Returns the queryKey.
     *
     * @return the queryKey
     */
    public String getQueryKey() {
        return queryKey;
    }

    /**
     * Sets the queryKey.
     *
     * @param queryKey
     *            the new queryKey to set
     */
    public void setQueryKey(String queryKey) {
        this.queryKey = queryKey;
    }

    /**
     * Returns the dataSourceUrlFile.
     *
     * @return the dataSourceUrlFile
     */
    public String getDataSourceUrlFile() {
        return dataSourceUrlFile;
    }

    /**
     * Sets the dataSourceUrlFile.
     *
     * @param dataSourceUrlFile
     *            the new dataSourceUrlFile to set
     */
    public void setDataSourceUrlFile(String dataSourceUrlFile) {
        this.dataSourceUrlFile = dataSourceUrlFile;
    }

    /**
     * Returns the dataSourceUrl.
     *
     * @return the dataSourceUrl
     */
    public String getDataSourceUrl() {
        return dataSourceUrl;
    }

    /**
     * Sets the dataSourceUrl.
     *
     * @param dataSourceUrl
     *            the new dataSourceUrl to set
     */
    public void setDataSourceUrl(String dataSourceUrl) {
        this.dataSourceUrl = dataSourceUrl;
    }
}
