/*
 * Copyright (C) 2012 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.bingmap.spatial.datasource;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.net.HttpURLConnection;
import java.net.URISyntaxException;
import java.text.MessageFormat;
import java.util.Locale;

import javax.annotation.PostConstruct;

import net.sf.ezmorph.object.DateMorpher;
import net.sf.json.util.JSONUtils;

import org.apache.http.HttpEntity;
import org.apache.http.HttpResponse;
import org.apache.http.client.ClientProtocolException;
import org.apache.http.client.HttpClient;
import org.apache.http.client.ResponseHandler;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.client.methods.HttpRequestBase;
import org.apache.http.client.utils.HttpClientUtils;
import org.apache.http.client.utils.URIBuilder;
import org.apache.http.entity.ByteArrayEntity;
import org.apache.http.impl.client.DefaultHttpClient;
import org.apache.http.util.EntityUtils;
import org.apache.log4j.Logger;

/**
 * <p>
 * This loader provides the functionality to upload data to Bing Spatial
 * DataSource and also to check the load job status.
 * </p>
 * <p>
 * This manager does not provide the functionality to search data from data
 * source because the searching could be done on client side directly with Bing
 * Maps AJAX Control.
 * </p>
 * <p>
 * <b>Thread Safety:</b> The ultimate thread safety depends on the thread safety
 * of Bing Spatial DataSource Management API. Without the insurance of thread
 * safety confirmed by Bing, this class should be used carefully. However since
 * the FARS data is uploaded yearly to spatial data source in practice, so this
 * should not be a problem.
 * </p>
 *
 * @author liuliquan, hanno
 * @version 1.0
 */
public class BingSpatialDataSourceManagerImpl implements BingSpatialDataSourceManager {
    /**
     * Logger.
     */
    private static final Logger LOGGER = Logger.getLogger(BingSpatialDataSourceManagerImpl.class);

    /**
     * URL of the spatial service.
     */
    private static final String SPATIAL_URL = "https://spatial.virtualearth.net/REST/v1/Dataflows/LoadDatasource/";

    /**
     * The URL parameter dataSourceName.
     */
    private static final String PARAM_DATA_SOURCE_NAME = "dataSourceName";

    /**
     * The URL parameter key.
     */
    private static final String PARAM_KEY = "key";

    /**
     * The URL parameter queryKey.
     */
    private static final String PARAM_QUERY_KEY = "queryKey";

    /**
     * The URL parameter loadOperation.
     */
    private static final String PARAM_LOAD_OPERATION = "loadOperation";

    /**
     * The URL parameter input.
     */
    private static final String PARAM_INPUT = "input";

    /**
     * The URL parameter output.
     */
    private static final String PARAM_OUTPUT = "output";

    /**
     * The URL parameter value json.
     */
    private static final String VALUE_JSON = "json";

    /**
     * Buffer size.
     */
    private static final int BUFFER_SIZE = 2048;

    /**
     * UTF-8 encoding.
     */
    private static final String UTF8 = "UTF-8";

    static {
        JSONUtils.getMorpherRegistry().registerMorpher(
            new DateMorpher(new String[] {"EEE, d MMM yyyy HH:mm:ss z" }, Locale.US, true));
    }

    /**
     * The configured data source.
     *
     * This field will be injected by Spring automatically.
     */
    private BingSpatialDataSource dataSource;

    /**
     * Empty constructor.
     */
    public BingSpatialDataSourceManagerImpl() {
        // empty
    }

    /**
     * Check bean is configured correctly.
     *
     * @throws BingSpatialConfigurationException
     *             if the bean is not configured correctly
     */
    @PostConstruct
    protected void checkInit() {
        BingSpatialHelper.checkBeanConfig("data source", this.dataSource);
    }

    /**
     * Create a Bing Spatial DataFlow job to load data into data source.
     *
     * <p>
     * This method creates a new data source if it does not exist, otherwise
     * depending on the given loadType:
     * <ul>
     * <li>If loadType is COMPLETE, the existing data will be replaced with the
     * newly uploaded data</li>
     * <li>If loadType is INCREMENTAL, then if the entity ID of a set of newly
     * uploaded data matches an existing entity ID in the data source, the
     * existing entity data is replaced by the new data.</li>
     * </ul>
     * If the new entity ID does not exist, a new entity is created.
     * </p>
     * <p>
     * Note this method returns once the load job is created, but it may not be
     * completed.
     * </p>
     * <p>
     * For more information on the call see <a
     * href="http://msdn.microsoft.com/en-us/library/gg585130.aspx">MS
     * documentation</a>
     * </p>
     *
     * @param data
     *            The data to be uploaded to Bing Spatial DataSource
     * @param loadType
     *            The load type
     * @throws IllegalArgumentException
     *             if any parameter is null
     * @throws BingSpatialDataSourceManagementException
     *             if any error occurs while loading data.
     */
    public void loadData(BingSpatialData data, BingSpatialDataLoadType loadType)
        throws BingSpatialDataSourceManagementException {
        try {
            BingSpatialHelper.checkValidity("data", data);
            BingSpatialHelper.checkValidity("loadType", loadType);
            checkDataSource();

            HttpPost request = createLoadDataRequest(data, loadType);
            String response = executeRequest(request, HttpURLConnection.HTTP_CREATED);

            BingSpatialResponseHandler handler = new BingSpatialResponseHandler(this);
            handler.handleLoadDataResponse(response);
        } catch (IllegalArgumentException e) {
            LOGGER.error(e.getMessage(), e);
            throw e;
        } catch (BingSpatialDataSourceManagementException e) {
            LOGGER.error(e.getMessage(), e);
            throw e;
        }
    }

    /**
     * Creates the loadData request.
     *
     * @param data
     *            data
     * @param loadType
     *            load type
     * @return loadData request
     * @throws BingSpatialDataSourceManagementException
     *             if an error occurs
     */
    private HttpPost createLoadDataRequest(BingSpatialData data, BingSpatialDataLoadType loadType)
        throws BingSpatialDataSourceManagementException {
        try {
            URIBuilder b = new URIBuilder(SPATIAL_URL);

            b.addParameter(PARAM_DATA_SOURCE_NAME, this.dataSource.getDataSourceName());
            b.addParameter(PARAM_KEY, this.getDataSource().getMasterKey());
            b.addParameter(PARAM_QUERY_KEY, this.dataSource.getQueryKey());
            b.addParameter(PARAM_LOAD_OPERATION, loadType.toString());
            b.addParameter(PARAM_INPUT, data.getFormat());
            b.addParameter(PARAM_OUTPUT, VALUE_JSON);

            HttpPost post = new HttpPost(b.build());
            post.setEntity(createEntity(data));

            return post;
        } catch (URISyntaxException e) {
            throw new BingSpatialDataSourceManagementException("Could not build request: " + e.getMessage(), e);
        } catch (IOException e) {
            throw new BingSpatialDataSourceManagementException("Could not build request: " + e.getMessage(), e);
        }
    }

    /**
     * Create HTTP entity for the spatial data. Because the Spatial service does
     * not support chunked transfer encoding we can not use a InputStreamEntity
     * and have to buffer the complete data before sending it to know the size
     * of the input stream of the spatial data before sending it.
     *
     * Therefore this method creates a ByteArrayEntity filled with the data.
     *
     * @param data
     *            spatial data
     * @return HTTP entity for the spatial data.
     * @throws IOException
     *             if an IO error occurs
     */
    private HttpEntity createEntity(BingSpatialData data) throws IOException {
        InputStream in = null;
        try {
            ByteArrayOutputStream bout = new ByteArrayOutputStream(BUFFER_SIZE);
            int read;
            byte[] buf = new byte[BUFFER_SIZE];
            in = data.getData();

            while ((read = in.read(buf)) != -1) {
                bout.write(buf, 0, read);
            }

            ByteArrayEntity bae = new ByteArrayEntity(bout.toByteArray());
            bae.setContentType(data.getMimeType());
            bae.setContentEncoding(UTF8);

            return bae;
        } finally {
            BingSpatialHelper.closeSilently(in);
        }
    }

    /**
     * Executes the given request.
     *
     * @param request
     *            HTTP request
     * @param expectedStatusCode
     *            the expected status code
     * @return the HTTP response content
     * @throws BingSpatialDataSourceManagementException
     *             if expectedStatusCode is not returned or if any other error
     *             occurs
     */
    private String executeRequest(HttpRequestBase request, final int expectedStatusCode)
        throws BingSpatialDataSourceManagementException {
        HttpClient client = null;

        try {
            client = new DefaultHttpClient();

            ResponseHandler<String> handler = new ResponseHandler<String>() {
                public String handleResponse(HttpResponse response) throws IOException {
                    int statusCode = response.getStatusLine().getStatusCode();

                    if (statusCode != expectedStatusCode) {
                        String details = new BingSpatialResponseHandler(BingSpatialDataSourceManagerImpl.this)
                            .getErrorDetails(getEntityString(response));
                        throw new ClientProtocolException(MessageFormat.format(
                            "Expected HTTP status code {0} but got {1} ({2}). Details: [{3}]", expectedStatusCode,
                            statusCode, response.getStatusLine().getReasonPhrase(), details));
                    }

                    return getEntityString(response);
                }

            };

            LOGGER.debug("Calling " + request);
            return client.execute(request, handler);
        } catch (IOException e) {
            throw new BingSpatialDataSourceManagementException("Could not execute HTTP request: " + e.getMessage(), e);
        } finally {
            HttpClientUtils.closeQuietly(client);
        }
    }

    /**
     * Get the content of the response as string.
     *
     * @param response
     *            HTTP response
     * @return the content of the response as string.
     * @throws IOException
     *             if an IO error occurs
     */
    private String getEntityString(HttpResponse response) throws IOException {
        HttpEntity entity = response.getEntity();
        if (entity != null) {
            return EntityUtils.toString(entity);
        } else {
            return "";
        }
    }

    /**
     * Checks the data source.
     *
     * @throws BingSpatialDataSourceManagementException
     *             if data source is not set or contains invalid data
     */
    private void checkDataSource() throws BingSpatialDataSourceManagementException {
        if (this.dataSource == null) {
            throw new BingSpatialDataSourceManagementException("No data source set, check the configuration");
        }
    }

    /**
     * Check the data source status.
     * <p>
     * For more information on the call see <a
     * href="http://msdn.microsoft.com/en-us/library/gg585135.aspx">MS
     * documentation</a>
     * </p>
     * <p>
     * For more information on the response see <a
     * href="http://msdn.microsoft.com/en-us/library/gg585127.aspx">MS
     * documentation</a>
     * </p>
     *
     * @return The data source status.
     * @throws BingSpatialDataSourceManagementException
     *             If any error occurs while checking the status.
     */
    public BingSpatialDataSourceStatus checkDataSourceStatus() throws BingSpatialDataSourceManagementException {
        try {
            checkDataSource();

            HttpGet request = createCheckDataSourceRequest();
            String response = executeRequest(request, HttpURLConnection.HTTP_OK);
            BingSpatialResponseHandler handler = new BingSpatialResponseHandler(this);

            return handler.handleCheckDataSourceStatusResponse(response);
        } catch (BingSpatialDataSourceManagementException e) {
            LOGGER.error(e.getMessage(), e);
            throw e;
        }
    }

    /**
     * Creates the checkDataSource request.
     *
     * @return checkDataSource request
     * @throws BingSpatialDataSourceManagementException
     *             if an error occurs
     */
    private HttpGet createCheckDataSourceRequest() throws BingSpatialDataSourceManagementException {
        try {
            BingSpatialProperties props = new BingSpatialProperties(this.dataSource);
            String jobUrl = props.getDataflowJobUrl();
            if (jobUrl == null) {
                throw new BingSpatialDataSourceManagementException("Dataflow job URL is not set");
            }

            URIBuilder b = new URIBuilder(jobUrl);
            b.addParameter(PARAM_KEY, this.getDataSource().getMasterKey());
            b.addParameter(PARAM_OUTPUT, VALUE_JSON);

            return new HttpGet(b.build());
        } catch (URISyntaxException e) {
            throw new BingSpatialDataSourceManagementException("Could not build request: " + e.getMessage(), e);
        } catch (IOException e) {
            throw new BingSpatialDataSourceManagementException("Could not build request: " + e.getMessage(), e);
        }
    }

    /**
     * Gets the Bing Spatial data source.
     *
     * @return the Bing Spatial data source.
     */
    public BingSpatialDataSource getDataSource() {
        return this.dataSource;
    }

    /**
     * Setter of dataSource. Called by Spring for injection.
     *
     * @param dataSource
     *            the data source
     */
    public void setDataSource(BingSpatialDataSource dataSource) {
        this.dataSource = dataSource;
    }
}
