/*
 * Copyright (C) 2012 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.bingmap.spatial.datasource;

import java.io.Closeable;
import java.io.IOException;
import java.text.MessageFormat;

/**
 * Package visible helper class.
 *
 * @author TCASSEMBLER
 * @version 1.0
 */
final class BingSpatialHelper {
    /**
     * Private empty constructor for helper class.
     */
    private BingSpatialHelper() {
        // empty
    }

    /**
     * Checks the given string value.
     *
     * @param name
     *            name of the value (used for exception message).
     * @param value
     *            value to be checked
     * @return the checked value
     * @throws IllegalArgumentException
     *             if value is null or empty
     */
    public static String checkValidity(String name, String value) {
        checkValidity(name, (Object) value);
        if (value.length() == 0) {
            throw new IllegalArgumentException(MessageFormat.format("{0} must not be empty", name));
        }
        return value;
    }

    /**
     * Checks the given value.
     *
     * @param name
     *            name of the value (used for exception message).
     * @param value
     *            value to be checked
     * @param <T>
     *            generic type of the checked value
     * @return the checked value
     * @throws IllegalArgumentException
     *             if value is null
     */
    public static <T> T checkValidity(String name, T value) {
        if (value == null) {
            throw new IllegalArgumentException(MessageFormat.format("{0} must not be null", name));
        }
        return value;
    }

    /**
     * Checks the given string value.
     *
     * @param name
     *            name of the value (used for exception message).
     * @param value
     *            value to be checked
     * @return the checked value
     * @throws BingSpatialConfigurationException
     *             if value is null or empty
     */
    public static String checkBeanConfig(String name, String value) {
        checkBeanConfig(name, (Object) value);
        if (value.length() == 0) {
            throw new BingSpatialConfigurationException(MessageFormat.format(
                "{0} must not be empty. Check the Spring context configuration.", name));
        }
        return value;
    }

    /**
     * Checks the given value.
     *
     * @param name
     *            name of the value (used for exception message).
     * @param value
     *            value to be checked
     * @param <T>
     *            generic type of the checked value
     * @return the checked value
     * @throws BingSpatialConfigurationException
     *             if value is null
     */
    public static <T> T checkBeanConfig(String name, T value) {
        if (value == null) {
            throw new BingSpatialConfigurationException(MessageFormat.format(
                "{0} must not be null. Check the Spring context configuration.", name));
        }
        return value;
    }

    /**
     * Closes the given {@link Closeable} ignoring all IO {@link Exception}s.
     *
     * @param c
     *            to be closed
     */
    public static void closeSilently(Closeable c) {
        if (c != null) {
            try {
                c.close();
            } catch (IOException e) {
                // ignore
            }
        }
    }
}
