/*
 * Copyright (C) 2012 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.bingmap.spatial.datasource;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.Properties;

/**
 * Properties file.
 *
 * @author hanno
 * @version 1.0
 */
class BingSpatialProperties {
    /**
     * Key to the data source url.
     */
    private static final String KEY_DATA_SOURCE_URL = "datasource";

    /**
     * Key to the job url.
     */
    private static final String KEY_JOB_URL = "job";

    /**
     * The data source.
     */
    private final BingSpatialDataSource ds;

    /**
     * File of properties.
     */
    private final File file;

    /**
     * Creates a new instance.
     *
     * @param ds
     *            data source
     */
    public BingSpatialProperties(BingSpatialDataSource ds) {
        this.ds = ds;
        this.file = new File(ds.getDataSourceUrlFile());
    }

    /**
     * Returns the URL to the dataflow job.
     *
     * @return the URL to the dataflow job
     * @throws IOException
     *             if an IO error occurs
     */
    public String getDataflowJobUrl() throws IOException {
        return load().getProperty(KEY_JOB_URL);
    }

    /**
     * Sets the URL to the dataflow job.
     *
     * @param url
     *            the URL to the dataflow job
     * @throws IOException
     *             if an IO error occurs
     */
    public void setDataflowJobUrl(String url) throws IOException {
        write(KEY_JOB_URL, url);
    }

    /**
     * Returns the URL to the data source.
     *
     * @return the URL to the data source.
     * @throws IOException
     *             if an IO error occurs
     */
    public String getDataSourceUrl() throws IOException {
        return load().getProperty(KEY_DATA_SOURCE_URL);
    }

    /**
     * Sets the URL to the data source.
     *
     * @param url
     *            the URL to the data source.
     * @throws IOException
     *             if an IO error occurs
     */
    public void setDataSourceUrl(String url) throws IOException {
        write(KEY_DATA_SOURCE_URL, url);
        this.ds.setDataSourceUrl(url);
    }

    /**
     * Loads properties from file.
     *
     * @return properties
     * @throws IOException
     *             if an IO error occurs
     */
    private Properties load() throws IOException {
        FileInputStream fin = null;

        try {
            Properties p = new Properties();

            if (this.file.exists()) {
                fin = new FileInputStream(this.file);
                // lock the file. The lock will be released on fin.close!
                fin.getChannel().lock(0, Long.MAX_VALUE, true);
                p.load(fin);
            }

            return p;
        } finally {
            if (fin != null) {
                fin.close();
            }
        }
    }

    /**
     * Write the given key.
     *
     * @param key
     *            key
     * @param value
     *            value
     * @throws IOException
     *             if an IO error occurs
     */
    private void write(String key, String value) throws IOException {
        FileOutputStream fout = null;
        try {
            Properties p = load();

            if (!this.file.exists()) {
                // ensure we can write the file
                this.file.getParentFile().mkdirs();
            }

            fout = new FileOutputStream(this.file);

            // lock the file. The lock will be released on fout.close!
            fout.getChannel().lock();

            p.setProperty(key, value);
            p.store(fout, null);
        } finally {
            if (fout != null) {
                fout.close();
            }
        }

    }

    /**
     * Checks if the underlying file has changed.
     *
     * @param d
     *            a timestamp
     * @return true if the underlying file has changed, false otherwise
     */
    public boolean hasChangedSince(long d) {
        return this.file.exists() && this.file.lastModified() > d;
    }

    /**
     * Returns the last modified timestamp.
     *
     * @return the last modified timestamp.
     */
    public long getLastModified() {
        return this.file.lastModified();
    }
}
