/*
 * Copyright (C) 2012 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.bingmap.spatial.datasource;

import java.io.IOException;

import net.sf.json.JSONArray;
import net.sf.json.JSONException;
import net.sf.json.JSONObject;
import net.sf.json.JSONSerializer;
import net.sf.json.JsonConfig;
import net.sf.json.util.PropertySetStrategy;

import org.apache.log4j.Logger;

/**
 * Parses a DataflowJob Response.
 *
 * <p>
 * <b>Thread safety:</b> This class is not thread safe but it is used in a
 * thread safe manner.
 * </p>
 *
 * @author hanno
 * @version 1.0
 */
class BingSpatialResponseHandler {
    /**
     * Logger.
     */
    private static final Logger LOGGER = Logger.getLogger(BingSpatialResponseHandler.class);

    /**
     * Status Completed. The dataflow job has completed. A status of completed
     * does not indicate success.
     */
    private static final String STATUS_COMPLETED = "Completed";

    /**
     * Status Pending. The dataflow job is processing.
     */
    private static final String STATUS_PENDING = "Pending";

    /**
     * Status Aborted. The workflow stopped because of an error.
     */
    private static final String STATUS_ABORTED = "Aborted";

    /**
     * The JSON field resourceSets.
     */
    private static final String FLD_RESOURCE_SETS = "resourceSets";

    /**
     * The JSON field resources.
     */
    private static final String FLD_RESOURCES = "resources";

    /**
     * The JSON field errorDetails.
     */
    private static final String FLD_ERROR_DETAILS = "errorDetails";

    /**
     * The JSON field links.
     */
    private static final String FLD_LINKS = "links";

    /**
     * The JSON field role.
     */
    private static final String FLD_ROLE = "role";

    /**
     * The JSON field url.
     */
    private static final String FLD_URL = "url";

    /**
     * Role datasource.
     */
    private static final String ROLE_DATASOURCE = "dataSource";

    /**
     * Role self.
     */
    private static final String ROLE_DATAFLOW_JOB = "self";

    /**
     * The root JSON object.
     */
    private JSONObject jobj;

    /**
     * Resources.
     */
    private JSONArray resources;

    /**
     * ResourceSets.
     */
    private JSONArray resourceSets;

    /**
     * Data source manager.
     */
    private final BingSpatialDataSourceManagerImpl dataSourceManager;

    /**
     * Creates a new instance.
     *
     * @param dataSourceManager
     *            data source manager
     */
    public BingSpatialResponseHandler(BingSpatialDataSourceManagerImpl dataSourceManager) {
        this.dataSourceManager = dataSourceManager;
    }

    /**
     * Returns the error details from the json.
     *
     * @param json
     *            json
     * @return the error details from the json.
     */
    public String getErrorDetails(String json) {
        StringBuilder b = new StringBuilder();

        if (json != null && json.length() != 0) {
            try {
                jobj = (JSONObject) JSONSerializer.toJSON(json);

                JSONArray ed = jobj.getJSONArray(FLD_ERROR_DETAILS);

                boolean isFirst = true;
                for (int i = 0; i < ed.size(); i++) {
                    if (isFirst) {
                        isFirst = false;
                    } else {
                        b.append(", ");
                    }
                    b.append(ed.getString(i));
                }
            } catch (JSONException e) {
                // ignore
            }
        }

        return b.toString();
    }

    /**
     * Parses the JSON from LoadData.
     *
     * @param json
     *            JSON string
     * @return The url to the dataflow job
     * @throws BingSpatialDataSourceManagementException
     *             if an error occurs
     */
    public String handleLoadDataResponse(String json) throws BingSpatialDataSourceManagementException {
        try {
            parseJson(json);

            String url = findUrlByRole(ROLE_DATAFLOW_JOB);

            LOGGER.info("DataflowJob created. URL is " + url);

            BingSpatialProperties props = new BingSpatialProperties(this.dataSourceManager.getDataSource());
            props.setDataflowJobUrl(url);

            return url;
        } catch (JSONException e) {
            throw new BingSpatialDataSourceManagementException("Could not parse response: " + e.getMessage(), e);
        } catch (IOException e) {
            throw new BingSpatialDataSourceManagementException("Could not parse response: " + e.getMessage(), e);
        }
    }

    /**
     * Parses the JSON from CheckDataSourceStatus.
     *
     * @param json
     *            JSON string
     * @return BingSpatialDataSourceStatus parsed from JSON
     * @throws BingSpatialDataSourceManagementException
     *             if an error occurs
     */
    public BingSpatialDataSourceStatus handleCheckDataSourceStatusResponse(String json)
        throws BingSpatialDataSourceManagementException {
        try {
            BingSpatialDataSourceStatus status = parseJson(json);

            LOGGER.info("DataflowJob status is " + status.getStatus());

            if (STATUS_COMPLETED.equals(status.getStatus())) {
                String url = findUrlByRole(ROLE_DATASOURCE);
                LOGGER.info("DataSource URL is " + url);

                BingSpatialProperties props = new BingSpatialProperties(this.dataSourceManager.getDataSource());
                props.setDataSourceUrl(url);
            }

            return status;
        } catch (JSONException e) {
            throw new BingSpatialDataSourceManagementException("Could not parse response: " + e.getMessage(), e);
        } catch (IOException e) {
            throw new BingSpatialDataSourceManagementException("Could not parse response: " + e.getMessage(), e);
        }
    }

    /**
     * Returns the URL with the given role from the first resource.
     *
     * @param role
     *            a role
     * @return the URL with the given role.
     * @throws BingSpatialDataSourceManagementException
     *             if not found
     */
    private String findUrlByRole(String role) throws BingSpatialDataSourceManagementException {
        JSONArray links = resources.getJSONObject(0).getJSONArray(FLD_LINKS);

        for (int i = 0; i < links.size(); i++) {
            JSONObject link = links.getJSONObject(i);
            if (role.equalsIgnoreCase(link.getString(FLD_ROLE))) {
                return link.getString(FLD_URL);
            }
        }

        throw new BingSpatialDataSourceManagementException("No URL found for role " + role);
    }

    /**
     * Create JSON configuration.
     *
     * @return JSON configuration.
     */
    private JsonConfig createJsonConfig() {
        JsonConfig jsonConfig = new JsonConfig();
        jsonConfig.setRootClass(BingSpatialDataSourceStatus.class);
        jsonConfig.setPropertySetStrategy(new PropertySetStrategy() {
            @Override
            public void setProperty(Object bean, String key, Object value) {
                try {
                    PropertySetStrategy.DEFAULT.setProperty(bean, key, value);
                } catch (JSONException e) {
                    // ignore if property is not found
                }
            }
        });
        return jsonConfig;
    }

    /**
     * Parses a BingSpatialDataSourceStatus from JSON.
     *
     * @param json
     *            json to be parsed
     * @return a BingSpatialDataSourceStatus from JSON.
     * @throws BingSpatialDataSourceManagementException
     *             if an error occurs
     */
    private BingSpatialDataSourceStatus parseJson(String json) throws BingSpatialDataSourceManagementException {
        jobj = (JSONObject) JSONSerializer.toJSON(json);
        resourceSets = jobj.getJSONArray(FLD_RESOURCE_SETS);
        if (resourceSets.isEmpty()) {
            throw new BingSpatialDataSourceManagementException("No resource sets found");
        }

        resources = resourceSets.getJSONObject(0).getJSONArray(FLD_RESOURCES);
        if (resources.isEmpty()) {
            throw new BingSpatialDataSourceManagementException("No resources found");
        }

        BingSpatialDataSourceStatus status;
        status = (BingSpatialDataSourceStatus) JSONSerializer.toJava(resources.getJSONObject(0), createJsonConfig());

        if (status == null) {
            throw new BingSpatialDataSourceManagementException("Resources array contains null value");
        }

        if (!STATUS_ABORTED.equals(status.getStatus()) && !STATUS_COMPLETED.equals(status.getStatus())
            && !STATUS_PENDING.equals(status.getStatus())) {
            throw new BingSpatialDataSourceManagementException("Unknown status: " + status.getStatus());
        }
        return status;
    }
}
