/*
 * Copyright (C) 2012 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.bingmap.spatial.datasource;

import java.io.IOException;

import javax.annotation.PostConstruct;

import org.apache.log4j.Logger;
import org.springframework.scheduling.annotation.Scheduled;

/**
 * This class provides a method that will be executed repeatedly by Spring
 * framework to read the fresh spatial data source URL from configured
 * dataSource.dataSourceUrlFile.
 *
 * <p>
 * <b>Thread Safety:</b> This class will change the state of configured
 * BingSpatialDataSource object, however since the FARS data is uploaded yearly
 * to spatial data source in practice, so the URL to query spatial data source
 * does not change in a long time.
 * </p>
 *
 * @author liuliquan, hanno
 * @version 1.0
 */
public class DataSourceUrlRefresher {
    /**
     * The logger.
     */
    private static final Logger LOGGER = Logger.getLogger(DataSourceUrlRefresher.class);

    /**
     * The fixed scheduled delay for hourly execution.
     */
    private static final long SCHEDULED_FIXED_DELAY_HOURLY = 3600000;

    /**
     * The configured data source.
     *
     * This field will be injected by Spring automatically.
     */
    private BingSpatialDataSource dataSource;

    /**
     * The last read time of configured dataSource.dataSourceUrlFile, in
     * milliseconds.
     */
    private long lastReadTime;

    /**
     * Empty constructor.
     */
    public DataSourceUrlRefresher() {
        // empty
    }

    /**
     * Check bean is configured correctly.
     *
     * @throws BingSpatialConfigurationException
     *             if the bean is not configured correctly
     */
    @PostConstruct
    protected void checkInit() {
        BingSpatialHelper.checkBeanConfig("data source", this.dataSource);
    }

    /**
     * Refreshes the data source URL from the configured
     * dataSource.dataSourceUrlFile.
     * <p>
     * Spring framework will execute this method automatically every hour.
     * </p>
     */
    @Scheduled(fixedDelay = SCHEDULED_FIXED_DELAY_HOURLY)
    public void refreshDataSourceUrl() {
        LOGGER.debug("Refreshing data source...");

        if (this.dataSource == null) {
            LOGGER.warn("No data source set. Check the configuration!");
            return;
        }

        try {
            refresh();
            LOGGER.debug("Refreshing data source done");
        } catch (IOException e) {
            LOGGER.warn("Could not refresh data source URL: " + e.getMessage(), e);
        }
    }

    /**
     * Performs the refresh.
     *
     * @throws IOException
     *             if an IO error occurs
     */
    private void refresh() throws IOException {
        BingSpatialProperties props = new BingSpatialProperties(this.dataSource);
        if (props.hasChangedSince(this.lastReadTime)) {
            this.lastReadTime = System.currentTimeMillis();
            this.dataSource.setDataSourceUrl(props.getDataSourceUrl());

            LOGGER.info("Refreshed DataSource URL is " + this.dataSource.getDataSourceUrl());
        }
    }

    /**
     * Setter of dataSource.
     *
     * Called by Spring for injection.
     *
     * @param dataSource
     *            the data source
     */
    public void setDataSource(BingSpatialDataSource dataSource) {
        this.dataSource = dataSource;
    }
}
