/*
 * Copyright (C) 2012 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.bingmap.spatial.datasource.fars;

import java.io.BufferedWriter;
import java.io.Closeable;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStreamWriter;
import java.text.MessageFormat;
import java.util.UUID;

import javax.annotation.PostConstruct;

import org.apache.log4j.Logger;
import org.apache.poi.openxml4j.exceptions.InvalidFormatException;
import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.usermodel.Workbook;
import org.apache.poi.ss.usermodel.WorkbookFactory;

import com.topcoder.bingmap.spatial.datasource.BingSpatialConfigurationException;
import com.topcoder.bingmap.spatial.datasource.BingSpatialData;
import com.topcoder.bingmap.spatial.datasource.BingSpatialDataParser;
import com.topcoder.bingmap.spatial.datasource.BingSpatialDataParsingException;
import com.topcoder.bingmap.spatial.datasource.BingSpatialDataSource;

/**
 * Implementation of BingSpatialDataParser to parse FARS data in excel format to
 * csv formatted Bing Spatial data.
 *
 * <p>
 * <b>Thread Safety:</b> This class is thread safe. The configured fields do not
 * change after injected.
 * </p>
 *
 * @author liuliquan, hanno
 * @version 1.0
 */
public class FARSExcelDataParser implements BingSpatialDataParser {
    /**
     * Logger.
     */
    private static final Logger LOGGER = Logger.getLogger(FARSExcelDataParser.class);

    /**
     * Number of columns in the excel file.
     */
    private static final int NUM_EXCEL_COLUMS = 8;

    /**
     * Index of column longitude.
     */
    private static final int COL_INDEX_LONGITUDE = 2;

    /**
     * CSV separator.
     */
    private static final String SEPARATOR = ",";

    /**
     * Cell value YES.
     */
    private static final String VALUE_YES = "YES";

    /**
     * Cell value NO.
     */
    private static final String VALUE_NO = "NO";

    /**
     * The csv header.
     */
    private static final String CSV_HEADER = "Bing Spatial Data Services, 1.0, {0}"
        + System.getProperty("line.separator") + "EntityID(Edm.String,primaryKey),Latitude(Edm.Double)"
        + ",Longitude(Edm.Double),AccidentType(Edm.String)";

    /**
     * The configured directory of the generated CSV file.
     *
     * This field will be injected by Spring automatically.
     */
    private String resultCSVFileDirectory;

    /**
     * The configured data source.
     *
     * This field will be injected by Spring automatically.
     */
    private BingSpatialDataSource dataSource;

    /**
     * Empty constructor.
     */
    public FARSExcelDataParser() {
        // empty
    }

    /**
     * Check bean is configured correctly.
     *
     * @throws BingSpatialConfigurationException
     *             if the bean is not configured correctly
     */
    @PostConstruct
    public void checkInit() {
        if (this.resultCSVFileDirectory == null || this.resultCSVFileDirectory.length() == 0) {
            throw new BingSpatialConfigurationException("Result CSV directory is not set.");
        }
        if (this.dataSource == null) {
            throw new BingSpatialConfigurationException("Datasource is not set");
        }
    }

    /**
     * Parse given input FARS data and produce csv formatted Bing Spatial data.
     *
     * The given input data must represent a valid Excel file and has format:
     * <ul>
     * <li>The columns are: Date, latitude, longitude, Pedestrian, Cyclist,
     * School, Drugs/Alcohol, All Other</li>
     * <li>The Date column can be ignored.</li>
     * <li>The latitude and longitude columns must have a valid double value.</li>
     * <li>The Pedestrian, Cyclist, School, Drugs/Alcohol, All Other columns
     * represent accident types, they must have a string value either "YES" or
     * "NO"</li>
     * </ul>
     * The output csv data is stored in configured resultCSVFileDirectory and
     * has format (with first lines describe the data schema as required by Bing
     * Spatial DataSource Management API):
     *
     * <pre>
     * Bing Spatial Data Services, 1.0, ${dataSource.getEntityTypeName()}
     * EntityID(Edm.String,primaryKey),Latitude(Edm.Double),Longitude(Edm.Double),AccidentType(Edm.String)
     * ef022265-ee48-45ec-8b37-c7bd1a0a9f56,20.40556944,150.8540084,10100
     * </pre>
     *
     * @param data
     *            the input data stream
     * @return The parsed Bing Spatial data in csv format
     * @throws IllegalArgumentException
     *             if given data stream is null.
     * @throws BingSpatialDataParsingException
     *             if any error occurs while parsing.
     */
    public BingSpatialData parseData(InputStream data) throws BingSpatialDataParsingException {
        try {
            if (data == null) {
                throw new IllegalArgumentException("Data must not be null");
            }

            if (dataSource == null) {
                throw new BingSpatialDataParsingException("No data source set.");
            }

            return this.saveAsCsv(data);
        } catch (IllegalArgumentException e) {
            LOGGER.error(e.getMessage(), e);
            throw e;
        } catch (BingSpatialDataParsingException e) {
            LOGGER.error(e.getMessage(), e);
            throw e;
        }
    }

    /**
     * Open an Excel workbook ready for conversion.
     *
     * @param data
     *            Excel file input
     * @return The parsed Bing Spatial data in csv format
     * @throws BingSpatialDataParsingException
     *             if an error occurs
     */
    private BingSpatialData saveAsCsv(InputStream data) throws BingSpatialDataParsingException {
        boolean dataWritten = false;
        try {
            BufferedWriter csvWriter = null;
            Workbook workbook = null;
            File csvFile = null;
            try {
                workbook = WorkbookFactory.create(data);

                csvFile = createCsvFile();
                csvWriter = new BufferedWriter(new OutputStreamWriter(new FileOutputStream(csvFile), "UTF-8"));

                csvWriter.write(MessageFormat.format(CSV_HEADER, dataSource.getEntityTypeName()));
                csvWriter.newLine();

                if (workbook.getNumberOfSheets() == 0) {
                    throw new BingSpatialDataParsingException("Excel file does not contain a sheet");
                }

                Sheet sheet = workbook.getSheetAt(0);
                if (sheet.getPhysicalNumberOfRows() > 0) {
                    int lastRowNum = sheet.getLastRowNum();
                    for (int j = 0; j <= lastRowNum; j++) {
                        dataWritten |= this.rowToCSV(sheet.getRow(j), csvWriter);
                    }
                }
            } finally {
                close(csvWriter);
                close(data);
            }

            if (!dataWritten) {
                throw new BingSpatialDataParsingException("Excel file did not contain any rows");
            }
            return new BingSpatialData(new FileInputStream(csvFile), "csv", "text/plain");
        } catch (InvalidFormatException e) {
            throw new BingSpatialDataParsingException("Could not save CSV: " + e.getMessage(), e);
        } catch (IOException e) {
            throw new BingSpatialDataParsingException("Could not save CSV: " + e.getMessage(), e);
        } catch (IllegalArgumentException e) {
            throw new BingSpatialDataParsingException("Could not save CSV: " + e.getMessage(), e);
        }
    }

    /**
     * Creates a new csv file.
     *
     * @return a new csv file.
     * @throws IOException
     *             if an IO error occurs
     */
    private File createCsvFile() throws IOException {
        File dir = new File(this.resultCSVFileDirectory);
        if (!dir.exists()) {
            dir.mkdirs();
        }

        File f = File.createTempFile("fars", "csv", dir);

        LOGGER.info("CSV file will be saved in " + f.getAbsolutePath());

        return f;
    }

    /**
     * Close.
     *
     * @param c
     *            to be closed
     */
    private void close(Closeable c) {
        if (c != null) {
            try {
                c.close();
            } catch (IOException e) {
                // ignore
            }
        }
    }

    /**
     * Called to convert a row of cells into a line of data in the CSV file.
     *
     * <pre>
     * EntityID(Edm.String,primaryKey),Latitude(Edm.Double),Longitude(Edm.Double),AccidentType(Edm.String)
     * </pre>
     *
     * @param csvWriter
     *            writer into the CSV file
     * @param row
     *            An instance of either the HSSFRow or XSSFRow classes
     * @return true if row has been written, false otherwise
     * @throws BingSpatialDataParsingException
     *             if a parsing error occurs
     * @throws IOException
     *             if an IO error occurs
     */
    private boolean rowToCSV(Row row, BufferedWriter csvWriter) throws BingSpatialDataParsingException, IOException {
        // ignore rows with no cells and the header
        if (row == null || row.getPhysicalNumberOfCells() == 0 || row.getRowNum() == 0) {
            return false;
        }

        boolean written = false;
        StringBuilder b = new StringBuilder();
        b.append(UUID.randomUUID()).append(SEPARATOR);

        for (int i = 1; i < NUM_EXCEL_COLUMS; i++) {
            Cell cell = row.getCell(i);
            if (!isEmpty(cell, row, written)) {
                written = true;
                if (i <= COL_INDEX_LONGITUDE) {
                    if (cell.getCellType() != Cell.CELL_TYPE_NUMERIC) {
                        throw new BingSpatialDataParsingException(MessageFormat.format(
                            "Cell {0} in row {1} is not numeric", i, row.getRowNum() + 1));
                    }
                    b.append(Double.toString(cell.getNumericCellValue())).append(SEPARATOR);
                } else {
                    if (cell.getCellType() != Cell.CELL_TYPE_STRING) {
                        throw new BingSpatialDataParsingException(MessageFormat.format(
                            "Cell {0} in row {1} is not a string", i, row.getRowNum() + 1));
                    }

                    String cellValue = cell.getStringCellValue();
                    if (!VALUE_NO.equalsIgnoreCase(cellValue) && !VALUE_YES.equalsIgnoreCase(cellValue)) {
                        throw new BingSpatialDataParsingException(MessageFormat.format(
                            "Cell {0} in row {1} contains invalid value: {2}", i, row.getRowNum() + 1, cellValue));
                    }

                    b.append(VALUE_YES.equalsIgnoreCase(cellValue) ? "1" : "0");
                }
            }
        }

        if (written) {
            csvWriter.write(b.toString());
            csvWriter.newLine();
        }

        return written;
    }

    /**
     * Check if the cell is empty.
     *
     * @param cell
     *            a cell
     * @param row
     *            a row
     * @param shouldBeFilled
     *            should the cell be filled
     * @return true if cell is empty, false otherwise.
     * @throws BingSpatialDataParsingException
     *             if cell is empty but should be filled
     */
    private boolean isEmpty(Cell cell, Row row, boolean shouldBeFilled) throws BingSpatialDataParsingException {
        boolean isEmpty = cell == null || cell.getCellType() == Cell.CELL_TYPE_BLANK;

        if (isEmpty && shouldBeFilled) {
            throw new BingSpatialDataParsingException(MessageFormat.format("Empty value in row {0}",
                row.getRowNum() + 1));
        }

        return isEmpty;
    }

    /**
     * Setter of result CSV Directory.
     * <p>
     * Called by Spring for injection.
     * </p>
     *
     * @param resultCSVFileDirectory
     *            the result CSV Directory
     */
    public void setResultCSVFileDirectory(String resultCSVFileDirectory) {
        this.resultCSVFileDirectory = resultCSVFileDirectory;
    }

    /**
     * Setter of data source.
     * <p>
     * Called by Spring for injection.
     * </p>
     *
     * @param dataSource
     *            the data source
     */
    public void setDataSource(BingSpatialDataSource dataSource) {
        this.dataSource = dataSource;
    }
}
