/*
 * Copyright (C) 2012 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.bingmap.mashup.controller.fars;

import java.io.IOException;
import java.io.InputStream;

import junit.framework.Assert;

import org.junit.After;
import org.junit.Before;
import org.junit.Test;
import org.mockito.ArgumentMatcher;
import org.mockito.Mock;
import org.mockito.Mockito;
import org.mockito.MockitoAnnotations;
import org.springframework.web.multipart.MultipartFile;

import com.topcoder.bingmap.spatial.datasource.BingSpatialConfigurationException;
import com.topcoder.bingmap.spatial.datasource.BingSpatialData;
import com.topcoder.bingmap.spatial.datasource.BingSpatialDataLoadType;
import com.topcoder.bingmap.spatial.datasource.BingSpatialDataParser;
import com.topcoder.bingmap.spatial.datasource.BingSpatialDataParsingException;
import com.topcoder.bingmap.spatial.datasource.BingSpatialDataSource;
import com.topcoder.bingmap.spatial.datasource.BingSpatialDataSourceManagementException;
import com.topcoder.bingmap.spatial.datasource.BingSpatialDataSourceManager;
import com.topcoder.bingmap.spatial.datasource.BingSpatialDataSourceStatus;

/**
 * Tests for FARSBingSpatialDataUploadController class.
 *
 * @author hanno
 * @version 1.0
 */
public class FARSBingSpatialDataUploadControllerTest {
    /**
     * Controller to be tested.
     */
    private FARSBingSpatialDataUploadController controller;

    /**
     * Data source manager mock.
     */
    @Mock
    private BingSpatialDataSourceManager dsManager;

    /**
     * Data source mock.
     */
    @Mock
    private BingSpatialDataSource dataSource;

    /**
     * Data parser mock.
     */
    @Mock
    private BingSpatialDataParser dataParser;

    /**
     * File mock.
     */
    @Mock
    private MultipartFile uploadFile;

    /**
     * Input stream mock.
     */
    @Mock
    private InputStream inputStream;

    /**
     * Source status mock.
     */
    @Mock
    private BingSpatialDataSourceStatus sourceStatus;

    /**
     * Test set up.
     *
     * @throws Exception
     *             to JUni
     */
    @Before
    public void setUp() throws Exception {
        MockitoAnnotations.initMocks(this);
        this.controller = new FARSBingSpatialDataUploadController();
        this.controller.setDataSourceManager(this.dsManager);
        this.controller.setDataParser(this.dataParser);
        this.controller.checkInit();

        Mockito.when(this.dsManager.getDataSource()).thenReturn(this.dataSource);
        Mockito.when(this.uploadFile.getInputStream()).thenReturn(this.inputStream);
    }

    /**
     * Clean up.
     */
    @After
    public void cleanUp() {
        this.controller = null;
        this.dsManager = null;
        this.dataSource = null;
        this.dataParser = null;
    }

    /**
     * getUploadStatus test.
     *
     * @throws Exception
     *             to JUnit
     */
    @Test
    public void getUploadStatus() throws Exception {
        Mockito.when(this.dsManager.checkDataSourceStatus()).thenReturn(this.sourceStatus);
        BingSpatialDataSourceStatus uploadStatus = this.controller.getUploadStatus();

        Assert.assertSame("source status", this.sourceStatus, uploadStatus);
    }

    /**
     * getUploadStatus test with error.
     *
     * @throws Exception
     *             to JUnit
     */
    @Test
    public void getUploadStatusWithError() throws Exception {
        Mockito.when(this.dsManager.checkDataSourceStatus()).thenThrow(
            new BingSpatialDataSourceManagementException("test"));
        BingSpatialDataSourceStatus uploadStatus = this.controller.getUploadStatus();

        Assert.assertEquals("source status", "ServerError", uploadStatus.getStatus());
    }

    /**
     * showUpload test.
     */
    @Test
    public void showUpload() {
        Assert.assertEquals("view", "upload", this.controller.showUpload());
    }

    /**
     * showUploadResult test.
     */
    @Test
    public void showUploadResult() {
        Assert.assertEquals("view", "uploadResult", this.controller.showUploadResult());
    }

    /**
     * Upload csv.
     *
     * @throws Exception
     *             to JUnit
     */
    @Test
    public void uploadSpatialDataCsv() throws Exception {
        Mockito.when(this.uploadFile.getOriginalFilename()).thenReturn("asdf.cSv");
        String view = this.controller.uploadSpatialData(this.uploadFile, BingSpatialDataLoadType.COMPLETE);

        Assert.assertEquals("view", "redirect:/uploadResult", view);
        Mockito.verify(this.dsManager).loadData(Mockito.argThat(isSpatialData("csv", "text/plain")),
            Mockito.eq(BingSpatialDataLoadType.COMPLETE));
    }

    /**
     * Upload xml.
     *
     * @throws Exception
     *             to JUnit
     */
    @Test
    public void uploadSpatialDataXml() throws Exception {
        Mockito.when(this.uploadFile.getOriginalFilename()).thenReturn("asdf.xml");
        String view = this.controller.uploadSpatialData(this.uploadFile, BingSpatialDataLoadType.INCREMENTAL);

        Assert.assertEquals("view", "redirect:/uploadResult", view);
        Mockito.verify(this.dsManager).loadData(Mockito.argThat(isSpatialData("xml", "text/xml")),
            Mockito.eq(BingSpatialDataLoadType.INCREMENTAL));
    }

    /**
     * Upload xls.
     *
     * @throws Exception
     *             to JUnit
     */
    @Test
    public void uploadSpatialDataXls() throws Exception {
        Mockito.when(this.uploadFile.getOriginalFilename()).thenReturn("asdf.xls");

        BingSpatialData data = new BingSpatialData(this.inputStream, "csv", "text/plain");
        Mockito.when(this.dataParser.parseData(Mockito.eq(this.inputStream))).thenReturn(data);

        String view = this.controller.uploadSpatialData(this.uploadFile, BingSpatialDataLoadType.INCREMENTAL);

        Assert.assertEquals("view", "redirect:/uploadResult", view);
        Mockito.verify(this.dsManager).loadData(Mockito.argThat(isSpatialData("csv", "text/plain")),
            Mockito.eq(BingSpatialDataLoadType.INCREMENTAL));
    }

    /**
     * Upload IO error.
     *
     * @throws Exception
     *             to JUnit
     */
    @Test(expected = BingSpatialDataParsingException.class)
    public void uploadSpatialDataIOError() throws Exception {
        Mockito.when(this.uploadFile.getOriginalFilename()).thenReturn("asdf.xml");
        Mockito.when(this.uploadFile.getInputStream()).thenThrow(new IOException("test"));

        this.controller.uploadSpatialData(this.uploadFile, BingSpatialDataLoadType.INCREMENTAL);
    }

    /**
     * Argument matcher for spatial data.
     *
     * @param format
     *            format
     * @param mimeType
     *            mime type
     * @return matcher
     */
    private ArgumentMatcher<BingSpatialData> isSpatialData(final String format, final String mimeType) {
        return new ArgumentMatcher<BingSpatialData>() {
            @Override
            public boolean matches(Object argument) {
                BingSpatialData data = (BingSpatialData) argument;
                return data.getFormat().equals(format) && data.getMimeType().equals(mimeType);
            }
        };
    }

    /**
     * CheckInit without data source manager.
     */
    @Test(expected = BingSpatialConfigurationException.class)
    public void checkInitNullDs() {
        this.controller.setDataSourceManager(null);
        this.controller.checkInit();
    }

    /**
     * CheckInit without parser.
     */
    @Test(expected = BingSpatialConfigurationException.class)
    public void checkInitNullParser() {
        this.controller.setDataParser(null);
        this.controller.checkInit();
    }
}
