/*
 * Copyright (C) 2012 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.bingmap.spatial.datasource;

import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.InputStream;

import org.junit.After;
import org.junit.Before;
import org.junit.Test;

/**
 * Unit tests for BingSpatialDataSourceManagerImpl class.
 *
 * @author hanno
 * @version 1.0
 */
public class BingSpatialDataSourceManagerImplTest {
    /**
     * To be tested.
     */
    private BingSpatialDataSourceManagerImpl impl;

    /**
     * Data.
     */
    private BingSpatialData data;

    /**
     * Input stream.
     */
    private InputStream input;

    /**
     * Data source.
     */
    private BingSpatialDataSource dataSource;

    /**
     * Temporary DataSourceUrlFile.
     */
    private File tempDataSourceUrlFile;

    /**
     * Test setup.
     *
     * @throws Exception
     *             to JUnit
     */
    @Before
    public void setUp() throws Exception {
        this.tempDataSourceUrlFile = TestHelper.createTempFileWithContent("some url");
        this.input = new ByteArrayInputStream("test".getBytes());
        this.dataSource = new BingSpatialDataSource();
        this.dataSource.setDataSourceName("test_ds~`!$^={}");
        this.dataSource.setDataSourceUrl("http://foo.com");
        this.dataSource.setDataSourceUrlFile(this.tempDataSourceUrlFile.getAbsolutePath());
        this.dataSource.setEntityTypeName("entity type name");
        this.dataSource.setMasterKey("master");
        this.dataSource.setQueryKey("query");

        this.impl = new BingSpatialDataSourceManagerImpl();
        this.impl.setDataSource(this.dataSource);
        this.data = new BingSpatialData(this.input, "csv", "text/plain");

        this.impl.checkInit();
    }

    /**
     * Clean up.
     *
     * @throws Exception
     *             to JUnit
     */
    @After
    public void cleanUp() throws Exception {
        this.impl = null;
        this.data = null;
        this.input.close();
        this.input = null;
        TestHelper.delete(this.tempDataSourceUrlFile);
        this.tempDataSourceUrlFile = null;
    }

    /**
     * loadData with null BingSpatialData.
     *
     * <table border="1">
     * <tr>
     * <th>Input</th>
     * <th>Expected</th>
     * </tr>
     * <tr>
     * <td>null BingSpatialData</td>
     * <td>IllegalArgumentException</td>
     * </tr>
     * </table>
     *
     * @throws Exception
     *             to JUnit
     */
    @Test(expected = IllegalArgumentException.class)
    public void loadDataNullBingSpatialData() throws Exception {
        this.impl.loadData(null, BingSpatialDataLoadType.COMPLETE);
    }

    /**
     * loadData with null BingSpatialDataLoadType.
     *
     * <table border="1">
     * <tr>
     * <th>Input</th>
     * <th>Expected</th>
     * </tr>
     * <tr>
     * <td>null BingSpatialDataLoadType</td>
     * <td>IllegalArgumentException</td>
     * </tr>
     * </table>
     *
     * @throws Exception
     *             to JUnit
     */
    @Test(expected = IllegalArgumentException.class)
    public void loadDataNullBingSpatialDataLoadType() throws Exception {
        this.impl.loadData(this.data, null);
    }

    /**
     * loadData with null url.
     *
     * <table border="1">
     * <tr>
     * <th>Input</th>
     * <th>Expected</th>
     * </tr>
     * <tr>
     * <td>null url</td>
     * <td>BingSpatialDataSourceManagementException</td>
     * </tr>
     * </table>
     *
     * @throws Exception
     *             to JUnit
     */
    @Test(expected = BingSpatialDataSourceManagementException.class)
    public void loadDataNullUrl() throws Exception {
        this.dataSource.setDataSourceUrl(null);
        this.impl.loadData(this.data, BingSpatialDataLoadType.COMPLETE);
    }

    /**
     * loadData with null data source.
     *
     * <table border="1">
     * <tr>
     * <th>Input</th>
     * <th>Expected</th>
     * </tr>
     * <tr>
     * <td>null data source</td>
     * <td>BingSpatialDataSourceManagementException</td>
     * </tr>
     * </table>
     *
     * @throws Exception
     *             to JUnit
     */
    @Test(expected = BingSpatialDataSourceManagementException.class)
    public void loadDataNullDataSource() throws Exception {
        this.impl.setDataSource(null);
        this.impl.loadData(this.data, BingSpatialDataLoadType.COMPLETE);
    }

    /**
     * loadData with invalid url.
     *
     * <table border="1">
     * <tr>
     * <th>Input</th>
     * <th>Expected</th>
     * </tr>
     * <tr>
     * <td>invalid url</td>
     * <td>BingSpatialDataSourceManagementException</td>
     * </tr>
     * </table>
     *
     * @throws Exception
     *             to JUnit
     */
    @Test(expected = BingSpatialDataSourceManagementException.class)
    public void loadDataInvalidUrl() throws Exception {
        this.impl.getDataSource().setDataSourceUrl("httpx://invalid///");
        this.impl.loadData(this.data, BingSpatialDataLoadType.COMPLETE);
    }

    /**
     * loadData with unknown host.
     *
     * <table border="1">
     * <tr>
     * <th>Input</th>
     * <th>Expected</th>
     * </tr>
     * <tr>
     * <td>unknown host</td>
     * <td>BingSpatialDataSourceManagementException</td>
     * </tr>
     * </table>
     *
     * @throws Exception
     *             to JUnit
     */
    @Test(expected = BingSpatialDataSourceManagementException.class)
    public void loadDataUnknownHost() throws Exception {
        this.impl.getDataSource().setDataSourceUrl("http://this.is.an.unkonwn.host.for.test.topcoder.com");
        this.impl.loadData(this.data, BingSpatialDataLoadType.COMPLETE);
    }

    /**
     * checkDataSourceStatus test.
     *
     * <table border="1">
     * <tr>
     * <th>Input</th>
     * <th>Expected</th>
     * </tr>
     * <tr>
     * <td>null datasource</td>
     * <td>BingSpatialDataSourceManagementException</td>
     * </tr>
     * </table>
     *
     * @throws Exception
     *             to JUnit
     */
    @Test(expected = BingSpatialDataSourceManagementException.class)
    public void checkDataSourceStatusNull() throws Exception {
        this.impl.setDataSource(null);
        this.impl.checkDataSourceStatus();
    }

    /**
     * checkDataSourceStatus test.
     *
     * <table border="1">
     * <tr>
     * <th>Input</th>
     * <th>Expected</th>
     * </tr>
     * <tr>
     * <td>invalid url</td>
     * <td>BingSpatialDataSourceManagementException</td>
     * </tr>
     * </table>
     *
     * @throws Exception
     *             to JUnit
     */
    @Test(expected = BingSpatialDataSourceManagementException.class)
    public void checkDataSourceStatusInvalidUrl() throws Exception {
        this.impl.getDataSource().setDataSourceUrl("httpy://fdsaj");
        this.impl.checkDataSourceStatus();
    }
}
