/*
 * Copyright (C) 2012 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.bingmap.spatial.datasource;

import java.io.File;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Locale;

import junit.framework.Assert;

import org.junit.After;
import org.junit.Before;
import org.junit.Test;

/**
 * Unit tests for BingSpatialResponseHandler class.
 *
 * @author hanno
 * @version 1.0
 */
public class BingSpatialResponseHandlerTest {
    /**
     * To be tested.
     */
    private BingSpatialResponseHandler parser;

    /**
     * Completed date.
     */
    private Date completedDate;

    /**
     * Created date.
     */
    private Date createdDate;

    /**
     * Data source.
     */
    private BingSpatialDataSource dataSource;

    /**
     * Temporary DataSourceUrlFile.
     */
    private File tempDataSourceUrlFile;

    /**
     * Test setup.
     *
     * @throws Exception
     *             to JUnit
     */
    @Before
    public void setUp() throws Exception {
        this.tempDataSourceUrlFile = TestHelper.createTempFileWithContent("some url");
        this.dataSource = new BingSpatialDataSource();
        this.dataSource.setDataSourceName("test_ds~`!$^={}");
        this.dataSource.setDataSourceUrl("http://foo.com");
        this.dataSource.setDataSourceUrlFile(this.tempDataSourceUrlFile.getAbsolutePath());
        this.dataSource.setEntityTypeName("entity type name");
        this.dataSource.setMasterKey("master");
        this.dataSource.setQueryKey("query");

        BingSpatialDataSourceManagerImpl impl = new BingSpatialDataSourceManagerImpl();
        impl.setDataSource(this.dataSource);
        this.parser = new BingSpatialResponseHandler(impl);

        SimpleDateFormat df = new SimpleDateFormat("EEE, d MMM yyyy HH:mm:ss z", Locale.US);
        df.setLenient(true);

        this.completedDate = df.parse("Mon, 10 May 2010 20:23:49 GMT");
        this.createdDate = df.parse("Mon, 10 May 2010 20:22:35 GMT");
    }

    /**
     * Clean up.
     *
     * @throws Exception
     *             to JUnit
     */
    @After
    public void cleanUp() throws Exception {
        this.parser = null;
        this.completedDate = null;
        this.createdDate = null;
        TestHelper.delete(this.tempDataSourceUrlFile);
        this.tempDataSourceUrlFile = null;
    }

    /**
     * Parse json response.
     *
     * <table border="1">
     * <tr>
     * <th>Input</th>
     * <th>Expected</th>
     * </tr>
     * <tr>
     * <td>Json with status Aborted</td>
     * <td>correct status object</td>
     * </tr>
     * </table>
     *
     * @throws Exception
     *             to JUnit
     */
    @Test
    public void parseJsonAborted() throws Exception {
        BingSpatialDataSourceStatus s = this.parser.handleCheckDataSourceStatusResponse(TestHelper
            .readFile("/checkDataSourceStatus_Aborted.json"));

        Assert.assertEquals("Status", "Aborted", s.getStatus());
        Assert.assertEquals("Error message", "test error message", s.getErrorMessage());
        Assert.assertEquals("Completed date", completedDate, s.getCompletedDate());
        Assert.assertEquals("Created date", createdDate, s.getCreatedDate());
        Assert.assertEquals("Failed entity count", 1, s.getFailedEntityCount());
        Assert.assertEquals("Processed entity count", 2, s.getProcessedEntityCount());
        Assert.assertEquals("Total entity count", 3, s.getTotalEntityCount());
    }

    /**
     * Parse json response.
     *
     * <table border="1">
     * <tr>
     * <th>Input</th>
     * <th>Expected</th>
     * </tr>
     * <tr>
     * <td>Json with status Pending</td>
     * <td>correct status object</td>
     * </tr>
     * </table>
     *
     * @throws Exception
     *             to JUnit
     */
    @Test
    public void parseJsonPending() throws Exception {
        BingSpatialDataSourceStatus s = this.parser.handleCheckDataSourceStatusResponse(TestHelper
            .readFile("/checkDataSourceStatus_Pending.json"));

        Assert.assertEquals("Status", "Pending", s.getStatus());
        Assert.assertEquals("Error message", null, s.getErrorMessage());
        Assert.assertEquals("Completed date", completedDate, s.getCompletedDate());
        Assert.assertEquals("Created date", createdDate, s.getCreatedDate());
        Assert.assertEquals("Failed entity count", 0, s.getFailedEntityCount());
        Assert.assertEquals("Processed entity count", 2, s.getProcessedEntityCount());
        Assert.assertEquals("Total entity count", 3, s.getTotalEntityCount());
    }

    /**
     * Parse json response.
     *
     * <table border="1">
     * <tr>
     * <th>Input</th>
     * <th>Expected</th>
     * </tr>
     * <tr>
     * <td>Json with status Completed</td>
     * <td>correct status object</td>
     * </tr>
     * </table>
     *
     * @throws Exception
     *             to JUnit
     */
    @Test
    public void parseJsonCompleted() throws Exception {
        BingSpatialDataSourceStatus s = this.parser.handleCheckDataSourceStatusResponse(TestHelper
            .readFile("/checkDataSourceStatus_Completed.json"));

        Assert.assertEquals("Status", "Completed", s.getStatus());
        Assert.assertEquals("Error message", null, s.getErrorMessage());
        Assert.assertEquals("Completed date", completedDate, s.getCompletedDate());
        Assert.assertEquals("Created date", createdDate, s.getCreatedDate());
        Assert.assertEquals("Failed entity count", 0, s.getFailedEntityCount());
        Assert.assertEquals("Processed entity count", 3, s.getProcessedEntityCount());
        Assert.assertEquals("Total entity count", 3, s.getTotalEntityCount());

        Assert.assertEquals("",
            "https://spatial.virtualearth.net/REST/v1/data/20181f26d9e94c81acdf9496133d4f23/FourthCoffeeSample",
            this.dataSource.getDataSourceUrl());
    }

    /**
     * Parse json response.
     *
     * <table border="1">
     * <tr>
     * <th>Input</th>
     * <th>Expected</th>
     * </tr>
     * <tr>
     * <td>Json with error response</td>
     * <td>correct error details</td>
     * </tr>
     * </table>
     *
     * @throws Exception
     *             to JUnit
     */
    @Test
    public void getErrorDetails() throws Exception {
        String s = this.parser.getErrorDetails(TestHelper.readFile("/errorResponse.json"));

        Assert.assertEquals("Error details", "first, second", s);
    }

    /**
     * Parse json response.
     *
     * <table border="1">
     * <tr>
     * <th>Input</th>
     * <th>Expected</th>
     * </tr>
     * <tr>
     * <td>Json with error response without details/td>
     * <td>empty error details</td>
     * </tr>
     * </table>
     *
     * @throws Exception
     *             to JUnit
     */
    @Test
    public void getErrorDetailsNoDetails() throws Exception {
        String s = this.parser.getErrorDetails(TestHelper.readFile("/errorResponse_NoDetails.json"));

        Assert.assertEquals("Error details", "", s);
    }

    /**
     * Parse json response.
     *
     * <table border="1">
     * <tr>
     * <th>Input</th>
     * <th>Expected</th>
     * </tr>
     * <tr>
     * <td>Json with status Foo</td>
     * <td>BingSpatialDataSourceManagementException</td>
     * </tr>
     * </table>
     *
     * @throws Exception
     *             to JUnit
     */
    @Test(expected = BingSpatialDataSourceManagementException.class)
    public void parseJsonUnknown() throws Exception {
        this.parser.handleCheckDataSourceStatusResponse(TestHelper.readFile("/checkDataSourceStatus_Unknown.json"));
    }

    /**
     * Parse json response.
     *
     * <table border="1">
     * <tr>
     * <th>Input</th>
     * <th>Expected</th>
     * </tr>
     * <tr>
     * <td>Json with empty resource sets</td>
     * <td>BingSpatialDataSourceManagementException</td>
     * </tr>
     * </table>
     *
     * @throws Exception
     *             to JUnit
     */
    @Test(expected = BingSpatialDataSourceManagementException.class)
    public void parseJsonEmptyResourceSets() throws Exception {
        this.parser.handleCheckDataSourceStatusResponse(TestHelper
            .readFile("/checkDataSourceStatus_EmptyResourceSets.json"));
    }

    /**
     * Parse json response.
     *
     * <table border="1">
     * <tr>
     * <th>Input</th>
     * <th>Expected</th>
     * </tr>
     * <tr>
     * <td>Json with empty resources</td>
     * <td>BingSpatialDataSourceManagementException</td>
     * </tr>
     * </table>
     *
     * @throws Exception
     *             to JUnit
     */
    @Test(expected = BingSpatialDataSourceManagementException.class)
    public void parseJsonEmptyResources() throws Exception {
        this.parser.handleCheckDataSourceStatusResponse(TestHelper
            .readFile("/checkDataSourceStatus_EmptyResources.json"));
    }

    /**
     * Parse json response.
     *
     * <table border="1">
     * <tr>
     * <th>Input</th>
     * <th>Expected</th>
     * </tr>
     * <tr>
     * <td>Json response/td>
     * <td>job url</td>
     * </tr>
     * </table>
     *
     * @throws Exception
     *             to JUnit
     */
    @Test
    public void handleLoadDataResponse() throws Exception {
        String s = this.parser.handleLoadDataResponse(TestHelper.readFile("/loadDataResponse.json"));

        Assert.assertEquals("Job URL",
            "https://spatial.virtualearth.net/REST/v1/dataflows/LoadDataSource/f8293dc72ac04942b7cb003c9608c547", s);
    }

    /**
     * Parse json response.
     *
     * <table border="1">
     * <tr>
     * <th>Input</th>
     * <th>Expected</th>
     * </tr>
     * <tr>
     * <td>Json response without url/td>
     * <td>BingSpatialDataSourceManagementException</td>
     * </tr>
     * </table>
     *
     * @throws Exception
     *             to JUnit
     */
    @Test(expected = BingSpatialDataSourceManagementException.class)
    public void handleLoadDataResponseNoJobUrl() throws Exception {
        this.parser.handleLoadDataResponse(TestHelper.readFile("/loadDataResponse_NoJobUrl.json"));
    }
}
