/*
 * Copyright (C) 2012 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.bingmap.spatial.datasource;

import java.io.File;
import java.io.IOException;

import junit.framework.Assert;

import org.junit.After;
import org.junit.Before;
import org.junit.Test;

/**
 * Unit tests for DataSourceUrlRefresher class.
 *
 * @author hanno
 * @version 1.0
 */
public class DataSourceUrlRefresherTest {
    /**
     * Test url.
     */
    private static final String URL = "test.foo/xyz.do";

    /**
     * To be tested.
     */
    private DataSourceUrlRefresher refresher;

    /**
     * Data source configuration.
     */
    private BingSpatialDataSource dataSource;

    /**
     * The data source url file.
     */
    private File dataSourceUrlFile;

    /**
     * Test setup.
     *
     * @throws IOException
     *             to JUnit.
     */
    @Before
    public void setUp() throws IOException {
        this.refresher = new DataSourceUrlRefresher();
        this.dataSourceUrlFile = TestHelper.createTempFileWithContent("datasource=" + URL);

        this.dataSource = new BingSpatialDataSource();
        this.dataSource.setDataSourceUrlFile(this.dataSourceUrlFile.getAbsolutePath());

        this.refresher.setDataSource(this.dataSource);
        this.refresher.checkInit();
    }

    /**
     * Clean up.
     */
    @After
    public void cleanUp() {
        TestHelper.delete(this.dataSourceUrlFile);
        this.dataSourceUrlFile = null;
        this.refresher = null;
    }

    /**
     * Test refresh with null data source.
     *
     * <table border="1">
     * <tr>
     * <th>Input</th>
     * <th>Expected</th>
     * </tr>
     * <tr>
     * <td>null data source</td>
     * <td>URL is not set</td>
     * </tr>
     * </table>
     */
    @Test
    public void refreshNulDataSource() {
        this.refresher.setDataSource(null);
        this.refresher.refreshDataSourceUrl();
        Assert.assertNull("data source URL", this.dataSource.getDataSourceUrl());
    }

    /**
     * Test refresh with existing file.
     *
     * <table border="1">
     * <tr>
     * <th>Input</th>
     * <th>Expected</th>
     * </tr>
     * <tr>
     * <td>existing file</td>
     * <td>URL is set</td>
     * </tr>
     * </table>
     */
    @Test
    public void refreshExistingFile() {
        Assert.assertNull("data source URL", this.dataSource.getDataSourceUrl());
        this.refresher.refreshDataSourceUrl();
        Assert.assertEquals("data source URL", URL, this.dataSource.getDataSourceUrl());
    }

    /**
     * Test refresh with existing file.
     *
     * <table border="1">
     * <tr>
     * <th>Input</th>
     * <th>Expected</th>
     * </tr>
     * <tr>
     * <td>existing file</td>
     * <td>URL is set</td>
     * </tr>
     * </table>
     */
    @Test
    public void refreshExistingFileTwoTimes() {
        Assert.assertNull("data source URL", this.dataSource.getDataSourceUrl());

        this.refresher.refreshDataSourceUrl();
        Assert.assertEquals("data source URL", URL, this.dataSource.getDataSourceUrl());

        this.refresher.refreshDataSourceUrl();
        Assert.assertEquals("data source URL", URL, this.dataSource.getDataSourceUrl());
    }

    /**
     * Test refresh with directory.
     *
     * <table border="1">
     * <tr>
     * <th>Input</th>
     * <th>Expected</th>
     * </tr>
     * <tr>
     * <td>directory</td>
     * <td>no exception is thrown, URL not set</td>
     * </tr>
     * </table>
     */
    @Test
    public void refreshDirectory() {
        this.dataSourceUrlFile.delete();
        this.dataSourceUrlFile = TestHelper.createTempDir("test");
        Assert.assertNull("data source URL", this.dataSource.getDataSourceUrl());

        this.refresher.refreshDataSourceUrl();
        Assert.assertNull("data source URL", this.dataSource.getDataSourceUrl());
    }

    /**
     * Test refresh with non existing file.
     *
     * <table border="1">
     * <tr>
     * <th>Input</th>
     * <th>Expected</th>
     * </tr>
     * <tr>
     * <td>non existing file</td>
     * <td>no exception is thrown, URL not set</td>
     * </tr>
     * </table>
     */
    @Test
    public void refreshNonExistingFile() {
        this.dataSourceUrlFile.delete();
        Assert.assertNull("data source URL", this.dataSource.getDataSourceUrl());

        this.refresher.refreshDataSourceUrl();
        Assert.assertNull("data source URL", this.dataSource.getDataSourceUrl());
    }
}
