/*
 * Copyright (C) 2012 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.bingmap.spatial.datasource.fars;

import java.io.File;

import junit.framework.Assert;

import org.junit.After;
import org.junit.Before;
import org.junit.Test;

import com.topcoder.bingmap.spatial.datasource.BingSpatialConfigurationException;
import com.topcoder.bingmap.spatial.datasource.BingSpatialData;
import com.topcoder.bingmap.spatial.datasource.BingSpatialDataParsingException;
import com.topcoder.bingmap.spatial.datasource.BingSpatialDataSource;
import com.topcoder.bingmap.spatial.datasource.TestHelper;

/**
 * Unit tests for FARSExcelDataParser class.
 *
 * @author hanno
 * @version 1.0
 */
public class FARSExcelDataParserTest {
    /**
     * To be tested.
     */
    private FARSExcelDataParser parser;

    /**
     * Temporary directory.
     */
    private File tempDir;

    /**
     * Data source.
     */
    private BingSpatialDataSource dataSource;

    /**
     * Test setup.
     */
    @Before
    public void setUp() {
        this.dataSource = new BingSpatialDataSource();
        this.dataSource.setDataSourceName("ds_name");
        this.dataSource.setEntityTypeName("entity_type_name");

        this.parser = new FARSExcelDataParser();
        this.parser.setDataSource(this.dataSource);
        this.tempDir = TestHelper.createTempDir("csv");
        this.parser.setResultCSVFileDirectory(this.tempDir.getAbsolutePath());
        this.parser.checkInit();
    }

    /**
     * Test clean up.
     */
    @After
    public void cleanUp() {
        this.parser = null;
        TestHelper.delete(this.tempDir);
        this.tempDir = null;
    }

    /**
     * parseData test.
     *
     * <table border="1">
     * <tr>
     * <th>Input</th>
     * <th>Expected</th>
     * </tr>
     * <tr>
     * <td>null Stream</td>
     * <td>IllegalArgumentException</td>
     * </tr>
     * </table>
     *
     * @throws Exception
     *             to JUnit
     */
    @Test(expected = IllegalArgumentException.class)
    public void parseDataNullStream() throws Exception {
        this.parser.parseData(null);
    }

    /**
     * parseData test.
     *
     * <table border="1">
     * <tr>
     * <th>Input</th>
     * <th>Expected</th>
     * </tr>
     * <tr>
     * <td>valid xls</td>
     * <td>correct csv</td>
     * </tr>
     * </table>
     *
     * @throws Exception
     *             to JUnit
     */
    @Test
    public void parseDataSampleSingleLine() throws Exception {
        BingSpatialData data = this.parser.parseData(getClass().getResourceAsStream("/SingleLine_FARS_Data.xls"));

        Assert.assertEquals("format", "csv", data.getFormat());
        Assert.assertEquals("type", "text/plain", data.getMimeType());
        String csv = TestHelper.readStream(data.getData());

        Assert.assertTrue("data",
            csv.endsWith(",36.35699167,-88.21834722,00001" + System.getProperty("line.separator")));
    }

    /**
     * parseData test.
     *
     * <table border="1">
     * <tr>
     * <th>Input</th>
     * <th>Expected</th>
     * </tr>
     * <tr>
     * <td>valid xlsx</td>
     * <td>correct csv</td>
     * </tr>
     * </table>
     *
     * @throws Exception
     *             to JUnit
     */
    @Test
    public void parseDataSampleSingleLineXlsx() throws Exception {
        BingSpatialData data = this.parser.parseData(getClass().getResourceAsStream("/SingleLine_FARS_Data.xlsx"));

        Assert.assertEquals("format", "csv", data.getFormat());
        Assert.assertEquals("type", "text/plain", data.getMimeType());
        String csv = TestHelper.readStream(data.getData());

        Assert.assertTrue("data",
            csv.endsWith(",36.35699167,-88.21834722,00001" + System.getProperty("line.separator")));
    }

    /**
     * parseData test.
     *
     * <table border="1">
     * <tr>
     * <th>Input</th>
     * <th>Expected</th>
     * </tr>
     * <tr>
     * <td>valid xls</td>
     * <td>correct csv</td>
     * </tr>
     * </table>
     *
     * @throws Exception
     *             to JUnit
     */
    @Test
    public void parseDataSample() throws Exception {
        BingSpatialData data = this.parser.parseData(getClass().getResourceAsStream("/Sample_FARS_Data.xls"));

        Assert.assertEquals("format", "csv", data.getFormat());
        Assert.assertEquals("type", "text/plain", data.getMimeType());
    }

    /**
     * parseData test.
     *
     * <table border="1">
     * <tr>
     * <th>Input</th>
     * <th>Expected</th>
     * </tr>
     * <tr>
     * <td>valid xlsx</td>
     * <td>correct csv</td>
     * </tr>
     * </table>
     *
     * @throws Exception
     *             to JUnit
     */
    @Test
    public void parseDataSampleXlsx() throws Exception {
        BingSpatialData data = this.parser.parseData(getClass().getResourceAsStream("/Sample_FARS_Data.xlsx"));

        Assert.assertEquals("format", "csv", data.getFormat());
        Assert.assertEquals("type", "text/plain", data.getMimeType());
    }

    /**
     * parseData test.
     *
     * <table border="1">
     * <tr>
     * <th>Input</th>
     * <th>Expected</th>
     * </tr>
     * <tr>
     * <td>InvalidDouble xlsx</td>
     * <td>BingSpatialDataParsingException</td>
     * </tr>
     * </table>
     *
     * @throws Exception
     *             to JUnit
     */
    @Test(expected = BingSpatialDataParsingException.class)
    public void parseInvalidDoubleXlsx() throws Exception {
        this.parser.parseData(getClass().getResourceAsStream("/SingleLine_FARS_InvalidDouble_Data.xlsx"));
    }

    /**
     * parseData test.
     *
     * <table border="1">
     * <tr>
     * <th>Input</th>
     * <th>Expected</th>
     * </tr>
     * <tr>
     * <td>InvalidAccidentData xlsx</td>
     * <td>BingSpatialDataParsingException</td>
     * </tr>
     * </table>
     *
     * @throws Exception
     *             to JUnit
     */
    @Test(expected = BingSpatialDataParsingException.class)
    public void parseInvalidAccidentDataXlsx() throws Exception {
        this.parser.parseData(getClass().getResourceAsStream("/SingleLine_FARS_InvalidAccidentData.xlsx"));
    }

    /**
     * parseData test.
     *
     * <table border="1">
     * <tr>
     * <th>Input</th>
     * <th>Expected</th>
     * </tr>
     * <tr>
     * <td>EmptyCell xlsx</td>
     * <td>BingSpatialDataParsingException</td>
     * </tr>
     * </table>
     *
     * @throws Exception
     *             to JUnit
     */
    @Test(expected = BingSpatialDataParsingException.class)
    public void parseEmptyCellXlsx() throws Exception {
        this.parser.parseData(getClass().getResourceAsStream("/SingleLine_FARS_EmptyCell_Data.xlsx"));
    }

    /**
     * checkInit test.
     *
     * <table border="1">
     * <tr>
     * <th>Input</th>
     * <th>Expected</th>
     * </tr>
     * <tr>
     * <td>null datasource</td>
     * <td>BingSpatialConfigurationException</td>
     * </tr>
     * </table>
     *
     * @throws Exception
     *             to JUnit
     */
    @Test(expected = BingSpatialConfigurationException.class)
    public void checkInitNullDS() throws Exception {
        this.parser.setDataSource(null);
        this.parser.checkInit();
    }

    /**
     * checkInit test.
     *
     * <table border="1">
     * <tr>
     * <th>Input</th>
     * <th>Expected</th>
     * </tr>
     * <tr>
     * <td>null ResultCSVFileDirectory</td>
     * <td>BingSpatialConfigurationException</td>
     * </tr>
     * </table>
     *
     * @throws Exception
     *             to JUnit
     */
    @Test(expected = BingSpatialConfigurationException.class)
    public void checkInitNullResultCSVFileDirectory() throws Exception {
        this.parser.setResultCSVFileDirectory(null);
        this.parser.checkInit();
    }
}
